@@ -17,48 +17,48 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <string.h>
-#include <strings.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <grp.h>
-#include <pwd.h>
-#include <errno.h>
-#ifdef HAVE_SOLARIS_ACLS
-#include <sys/acl.h>
-#endif
-#ifdef HAVE_FREEBSD_SUNACL
-#include <sunacl.h>
-#endif
-#ifdef HAVE_POSIX_ACLS
-#include <sys/acl.h>
-#endif
 #ifdef HAVE_ACL_LIBACL_H
 #include <acl/libacl.h>
 #endif
+#include <errno.h>
+#include <grp.h>
+#include <pwd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <strings.h>
+
+#ifdef HAVE_FREEBSD_SUNACL
+#include <sunacl.h>
+#endif
+
+#if defined(HAVE_POSIX_ACLS) || defined(HAVE_SOLARIS_ACLS)
+#include <sys/acl.h>
+#endif
 
-#include <atalk/errchk.h>
-#include <atalk/adouble.h>
-#include <atalk/vfs.h>
-#include <atalk/afp.h>
-#include <atalk/util.h>
-#include <atalk/cnid.h>
-#include <atalk/logger.h>
-#include <atalk/uuid.h>
 #include <atalk/acl.h>
-#include <atalk/bstrlib.h>
+#include <atalk/adouble.h>
+#include <atalk/afp.h>
 #include <atalk/bstradd.h>
-#include <atalk/unix.h>
+#include <atalk/bstrlib.h>
+#include <atalk/cnid.h>
+#include <atalk/errchk.h>
+#include <atalk/logger.h>
 #include <atalk/netatalk_conf.h>
+#include <atalk/unix.h>
+#include <atalk/util.h>
+#include <atalk/uuid.h>
+#include <atalk/vfs.h>
 
-#include "directory.h"
+#include "acl_mappings.h"
+#include "acls.h"
+#include "auth.h"
 #include "desktop.h"
-#include "volume.h"
+#include "directory.h"
 #include "fork.h"
 #include "unix.h"
-#include "acls.h"
-#include "acl_mappings.h"
-#include "auth.h"
+#include "volume.h"
+
 
 /* for map_acl() */
 #define SOLARIS_2_DARWIN       1
@@ -81,7 +81,7 @@
 
 #ifdef HAVE_NFSV4_ACLS
 
-/*! 
+/*!
  * Compile access rights for a user to one file-system object
  *
  * This combines combines all access rights for a user to one fs-object and
@@ -419,7 +419,7 @@ EC_CLEANUP:
     return rights;
 }
 
-/*! 
+/*!
  * Compile access rights for a user to one file-system object
  *
  * This combines combines all access rights for a user to one fs-object and
@@ -531,9 +531,9 @@ EC_CLEANUP:
  *
  * @returns         access rights
  */
-static u_char acl_permset_to_uarights(acl_entry_t entry) {
+static uint8_t acl_permset_to_uarights(acl_entry_t entry) {
     acl_permset_t permset;
-    u_char rights = 0;
+    uint8_t rights = 0;
 
     if (acl_get_permset(entry, &permset) == -1)
         return rights;
@@ -589,9 +589,9 @@ static int posix_acls_to_uaperms(const AFPObj *obj, const char *path, struct sta
     gid_t *gid;
     uid_t whoami = geteuid();
 
-    u_char group_rights = 0x00;
-    u_char acl_rights = 0x00;
-    u_char mask = 0xff;
+    uint8_t group_rights = 0x00;
+    uint8_t acl_rights = 0x00;
+    uint8_t mask = 0xff;
 
     acl = acl_get_file(path, ACL_TYPE_ACCESS);
     if (acl == NULL) {
@@ -728,7 +728,7 @@ static acl_perm_t map_darwin_right_to_posix_permset(uint32_t darwin_ace_rights,
  * Add a permission of "type" for user or group "id" to an ACL. Scan the ACL
  * for existing permissions for this type/id, if there is one add the perm,
  * otherwise create a new ACL entry.
- * perm can be or'ed ACL_READ, ACL_WRITE and ACL_EXECUTE.  
+ * perm can be or'ed ACL_READ, ACL_WRITE and ACL_EXECUTE.
  *
  * @param aclp     (rw) pointer to ACL
  * @param type     (r)  acl_tag_t of ACL_USER or ACL_GROUP
@@ -1407,21 +1407,6 @@ static int check_acl_access(const AFPObj *obj,
     AFP_ASSERT(vol);
 
     /* This check is not used anymore, as OS X Server seems to be ignoring too */
-#if 0
-    /* Get uid or gid from UUID */
-    EC_ZERO_ERR(getnamefromuuid(uuid, &username, &uuidtype), AFPERR_PARAM);
-    switch (uuidtype) {
-    case UUID_USER:
-        break;
-    case UUID_GROUP:
-        LOG(log_warning, logtype_afpd, "check_acl_access: afp_access not supported for groups");
-        EC_STATUS(AFPERR_MISC);
-        goto EC_CLEANUP;
-    default:
-        EC_STATUS(AFPERR_MISC);
-        goto EC_CLEANUP;
-    }
-#endif
 
     EC_ZERO_LOG_ERR(ostat(path, &st, vol_syml_opt(vol)), AFPERR_PARAM);
 
@@ -1737,12 +1722,20 @@ int afp_setacl(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_, size
     /* Change ACL ? */
     if (bitmap & kFileSec_ACL) {
         LOG(log_debug, logtype_afpd, "afp_setacl: Change ACL request.");
+        size_t ibuflen0 = ibuflen;
+        const char *ibuf0 = ibuf;
+
         /*  Get no of ACEs the client put on the wire */
         uint32_t ace_count;
         memcpy(&ace_count, ibuf, sizeof(uint32_t));
         ace_count = htonl(ace_count);
         ibuf += 8;      /* skip ACL flags (see acls.h) */
 
+        if (ibuf + ace_count * sizeof(darwin_ace_t) > ibuf0 + ibuflen0){
+            LOG(log_error, logtype_afpd, "afp_setacl: ace_count is too big");
+            return AFPERR_MISC;
+        }
+
         ret = set_acl(vol,
                       s_path->u_name,
                       (bitmap & kFileSec_Inherit),
