@@ -1,4 +1,4 @@
-/* 
+/*
  * Netatalk 2002 (c)
  * Copyright (C) 1990, 1993 Regents of The University of Michigan
  * Copyright (C) 2010 Frank Lahm
@@ -21,45 +21,44 @@
  *
  * Starting with Netatalk 2.2 searching by name criteria utilizes the
  * CNID database in conjunction with an enhanced cnid_dbd. This requires
- * the use of cnidscheme:dbd for the searched volume, the new functionality
- * is not built into cnidscheme:cdb.
+ * the use of cnidscheme:dbd for the searched volume.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
+#include <ctype.h>
+#include <errno.h>
+#include <netinet/in.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <errno.h>
-#include <ctype.h>
 #include <string.h>
-#include <time.h>
 #include <string.h>
 #include <sys/file.h>
-#include <netinet/in.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <time.h>
+#include <unistd.h>
 
-#include <atalk/afp.h>
 #include <atalk/adouble.h>
-#include <atalk/logger.h>
-#include <atalk/cnid.h>
-#include <atalk/cnid_bdb_private.h>
-#include <atalk/util.h>
+#include <atalk/afp.h>
 #include <atalk/bstradd.h>
-#include <atalk/unicode.h>
+#include <atalk/cnid_bdb_private.h>
+#include <atalk/cnid.h>
 #include <atalk/globals.h>
+#include <atalk/logger.h>
 #include <atalk/netatalk_conf.h>
+#include <atalk/unicode.h>
+#include <atalk/util.h>
 
 #include "desktop.h"
-#include "directory.h"
 #include "dircache.h"
+#include "directory.h"
 #include "file.h"
-#include "volume.h"
 #include "filedir.h"
 #include "fork.h"
+#include "volume.h"
 
 
 struct finderinfo {
@@ -99,7 +98,7 @@ struct scrit {
 	uint32_t pdid;             /* Parent DID */
     uint16_t offcnt;           /* Offspring count */
 	struct finderinfo finfo;    /* Finder info */
-	char lname[64];             /* Long name */ 
+	char lname[64];             /* Long name */
 	char utf8name[514];         /* UTF8 or UCS2 name */ /* for convert_charset dest_len parameter +2 */
 };
 
@@ -112,10 +111,10 @@ struct scrit {
  *
  */
 struct dsitem {
-    cnid_t ds_did;         /* CNID of this directory           */
-    int    ds_checked;     /* Have we checked this directory ? */
+    cnid_t ds_did;               /* CNID of this directory           */
+    uint32_t ds_checked;    /* Have we checked this directory ? */
 };
- 
+
 
 #define DS_BSIZE 128
 static int save_cidx = -1; /* Saved index of currently scanned directory. */
@@ -125,7 +124,7 @@ static int dsidx = 0;   	     /* First free item index... */
 static struct scrit c1, c2;          /* search criteria */
 
 /* Clears directory stack. */
-static void clearstack(void) 
+static void clearstack(void)
 {
 	save_cidx = -1;
 	while (dsidx > 0) {
@@ -134,7 +133,7 @@ static void clearstack(void)
 }
 
 /* Puts new item onto directory stack. */
-static int addstack(char *uname, struct dir *dir, int pidx)
+static int addstack(char *uname _U_, struct dir *dir, int pidx _U_)
 {
 	struct dsitem *ds;
     struct dsitem *tmpds = NULL;
@@ -142,7 +141,7 @@ static int addstack(char *uname, struct dir *dir, int pidx)
 	/* check if we have some space on stack... */
 	if (dsidx >= dssize) {
 		dssize += DS_BSIZE;
-		tmpds = realloc(dstack, dssize * sizeof(struct dsitem));	
+		tmpds = realloc(dstack, dssize * sizeof(struct dsitem));
 		if (tmpds == NULL) {
             clearstack();
             free(dstack);
@@ -174,23 +173,23 @@ static int reducestack(void)
 //			free(dstack[dsidx].path);
 		} else
 			return dsidx - 1;
-	} 
+	}
 	return -1;
-} 
+}
 
-/* Looks up for an opened adouble structure, opens resource fork of selected file. 
+/* Looks up for an opened adouble structure, opens resource fork of selected file.
  * FIXME What about noadouble?
 */
 static struct adouble *adl_lkup(struct vol *vol, struct path *path, struct adouble *adp)
 {
 	static struct adouble ad;
-	
+
 	struct ofork *of;
 	int isdir;
-	
+
 	if (adp)
 	    return adp;
-	    
+
 	isdir  = S_ISDIR(path->st.st_mode);
 
 	if (!isdir && (of = of_findname(vol, path))) {
@@ -198,13 +197,13 @@ static struct adouble *adl_lkup(struct vol *vol, struct path *path, struct adoub
 	} else {
 		ad_init(&ad, vol);
 		adp = &ad;
-	} 
+	}
 
     if ( ad_metadata( path->u_name, ((isdir) ? ADFLAGS_DIR : 0), adp) < 0 ) {
         adp = NULL; /* FIXME without resource fork adl_lkup will be call again */
     }
-    
-	return adp;	
+
+	return adp;
 }
 
 /* -------------------- */
@@ -226,7 +225,7 @@ unpack_finderinfo(struct vol *vol, struct path *path, struct adouble **adp, stru
 {
 	packed_finder  buf;
 	void           *ptr;
-	
+
     *adp = adl_lkup(vol, path, *adp);
 	ptr = get_finderinfo(vol, path->u_name, *adp, &buf,islnk);
 	return unpack_buffer(finfo, ptr);
@@ -237,7 +236,7 @@ unpack_finderinfo(struct vol *vol, struct path *path, struct adouble **adp, stru
 /* Criteria checker. This function returns a 2-bit value. */
 /* bit 0 means if checked file meets given criteria. */
 /* bit 1 means if it is a directory and we should descent into it. */
-/* uname - UNIX name 
+/* uname - UNIX name
  * fname - our fname (translated to UNIX)
  * cidx - index in directory stack
  */
@@ -261,16 +260,16 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if (!c1.fbitmap)
 			return 0;
 
-		/* compute the Mac name 
+		/* compute the Mac name
 		 * first try without id (it's slow to find it)
-		 * An other option would be to call get_id in utompath but 
+		 * An other option would be to call get_id in utompath but
 		 * we need to pass parent dir
 		*/
         if (!(path->m_name = utompath(vol, path->u_name, 0 , utf8_encoding(vol->v_obj)) )) {
         	/*retry with the right id */
-       
+
         	cnid_t id;
-        	
+
         	adp = adl_lkup(vol, path, adp);
         	id = get_id(vol, adp, &path->st, path->d_dir->d_did, path->u_name, strlen(path->u_name));
         	if (!id) {
@@ -284,38 +283,38 @@ static int crit_check(struct vol *vol, struct path *path) {
         	}
         }
 	}
-		
-	/* Kind of optimization: 
+
+	/* Kind of optimization:
 	 * -- first check things we've already have - filename
 	 * -- last check things we get from ad_open()
 	 * FIXME strmcp strstr (icase)
 	 */
 
 	/* Check for filename */
-	if ((c1.rbitmap & (1<<DIRPBIT_LNAME))) { 
+	if ((c1.rbitmap & (1U<<DIRPBIT_LNAME))) {
 		if ( (size_t)(-1) == (len = convert_string(vol->v_maccharset, CH_UCS2, path->m_name, -1, convbuf, 512)) )
 			goto crit_check_ret;
 
-		if ((c1.rbitmap & (1<<CATPBIT_PARTIAL))) {
+		if ((c1.rbitmap & (1U<<CATPBIT_PARTIAL))) {
 			if (strcasestr_w( (ucs2_t*) convbuf, (ucs2_t*) c1.lname) == NULL)
 				goto crit_check_ret;
 		} else
 			if (strcasecmp_w((ucs2_t*) convbuf, (ucs2_t*) c1.lname) != 0)
 				goto crit_check_ret;
-	} 
-	
-	if ((c1.rbitmap & (1<<FILPBIT_PDINFO))) { 
+	}
+
+	if ((c1.rbitmap & (1U<<FILPBIT_PDINFO))) {
 		if ( (size_t)(-1) == (len = convert_charset( CH_UTF8_MAC, CH_UCS2, CH_UTF8, path->m_name, strlen(path->m_name), convbuf, 512, &flags))) {
 			goto crit_check_ret;
 		}
 
-		if (c1.rbitmap & (1<<CATPBIT_PARTIAL)) {
+		if (c1.rbitmap & (1U<<CATPBIT_PARTIAL)) {
 			if (strcasestr_w((ucs2_t *) convbuf, (ucs2_t*)c1.utf8name) == NULL)
 				goto crit_check_ret;
 		} else
 			if (strcasecmp_w((ucs2_t *)convbuf, (ucs2_t*)c1.utf8name) != 0)
 				goto crit_check_ret;
-	} 
+	}
 
 
 	/* FIXME */
@@ -327,13 +326,13 @@ static int crit_check(struct vol *vol, struct path *path) {
 		c2.bdate = 0x7fffffff;
 
 	/* Check for modification date */
-	if ((c1.rbitmap & (1<<DIRPBIT_MDATE))) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_MDATE))) {
 		if (path->st.st_mtime < c1.mdate || path->st.st_mtime > c2.mdate)
 			goto crit_check_ret;
 	}
-	
+
 	/* Check for creation date... */
-	if ((c1.rbitmap & (1<<DIRPBIT_CDATE))) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_CDATE))) {
 		c_date = path->st.st_mtime;
 		adp = adl_lkup(vol, path, adp);
 		if (adp && ad_getdate(adp, AD_DATE_CREATE, &ac_date) >= 0)
@@ -344,7 +343,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 	}
 
 	/* Check for backup date... */
-	if ((c1.rbitmap & (1<<DIRPBIT_BDATE))) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_BDATE))) {
 		b_date = path->st.st_mtime;
 		adp = adl_lkup(vol, path, adp);
 		if (adp && ad_getdate(adp, AD_DATE_BACKUP, &ab_date) >= 0)
@@ -353,35 +352,35 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if (b_date < c1.bdate || b_date > c2.bdate)
 			goto crit_check_ret;
 	}
-				
+
 	/* Check attributes */
-	if ((c1.rbitmap & (1<<DIRPBIT_ATTR)) && c2.attr != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_ATTR)) && c2.attr != 0) {
 		if ((adp = adl_lkup(vol, path, adp))) {
 			ad_getattr(adp, &attr);
 			if ((attr & c2.attr) != c1.attr)
 				goto crit_check_ret;
-		} else 
+		} else
 			goto crit_check_ret;
-	}		
+	}
 
         /* Check file type ID */
-	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.f_type != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.f_type != 0) {
 	    finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		if (finfo->f_type != c1.finfo.f_type)
 			goto crit_check_ret;
 	}
-	
+
 	/* Check creator ID */
-	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.creator != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.creator != 0) {
 		if (!finfo) {
 			finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		}
 		if (finfo->creator != c1.finfo.creator)
 			goto crit_check_ret;
 	}
-		
+
 	/* Check finder info attributes */
-	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.attrs != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.attrs != 0) {
 		if (!finfo) {
 			finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		}
@@ -389,24 +388,24 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if ((finfo->attrs & c2.finfo.attrs) != c1.finfo.attrs)
 			goto crit_check_ret;
 	}
-	
+
 	/* Check label */
-	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.label != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.label != 0) {
 		if (!finfo) {
 			finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		}
 		if ((finfo->label & c2.finfo.label) != c1.finfo.label)
 			goto crit_check_ret;
-	}	
+	}
 	/* FIXME: Attributes check ! */
-	
+
 	/* All criteria are met. */
 	result |= 1;
 crit_check_ret:
 	if (adp != NULL)
 		ad_close(adp, ADFLAGS_HF);
 	return result;
-}  
+}
 
 /* ------------------------------ */
 static int rslt_add (const AFPObj *obj, struct vol *vol, struct path *path, char **buf, int ext)
@@ -416,11 +415,11 @@ static int rslt_add (const AFPObj *obj, struct vol *vol, struct path *path, char
 	int 		ret;
 	size_t		tbuf =0;
 	uint16_t	resultsize;
-	int 		isdir = S_ISDIR(path->st.st_mode); 
+	int 		isdir = S_ISDIR(path->st.st_mode);
 
 	/* Skip resultsize */
 	if (ext) {
-		p += sizeof(resultsize); 
+		p += sizeof(resultsize);
 	}
 	else {
 		p++;
@@ -430,9 +429,9 @@ static int rslt_add (const AFPObj *obj, struct vol *vol, struct path *path, char
 	if (ext) {
 		*p++ = 0;                  /* Pad */
 	}
-	
+
 	if ( isdir ) {
-        ret = getdirparams(obj, vol, c1.dbitmap, path, path->d_dir, p , &tbuf ); 
+        ret = getdirparams(obj, vol, c1.dbitmap, path, path->d_dir, p , &tbuf );
 	}
 	else {
 	    /* FIXME slow if we need the file ID, we already know it, done ? */
@@ -459,8 +458,8 @@ static int rslt_add (const AFPObj *obj, struct vol *vol, struct path *path, char
 	}
 
 	return 1;
-} 
-	
+}
+
 #define VETO_STR \
         "./../.AppleDouble/.AppleDB/Network Trash Folder/TheVolumeSettingsFolder/TheFindByContentFolder/.AppleDesktop/.Parent/"
 
@@ -481,7 +480,7 @@ static int rslt_add (const AFPObj *obj, struct vol *vol, struct path *path, char
 #define NUM_ROUNDS 200
 static int catsearch(const AFPObj *obj,
                      struct vol *vol,
-                     struct dir *dir,  
+                     struct dir *dir,
                      int rmatches,
                      uint32_t *pos,
                      char *rbuf,
@@ -519,8 +518,8 @@ static int catsearch(const AFPObj *obj,
 		if (dirpos != NULL) {
 			closedir(dirpos);
 			dirpos = NULL;
-		} 
-		
+		}
+
 		if (addstack(vpath, dir, -1) == -1) {
 			result = AFPERR_MISC;
 			goto catsearch_end;
@@ -533,7 +532,7 @@ static int catsearch(const AFPObj *obj,
         result = AFPERR_MISC;
         goto catsearch_end;
     }
-	
+
 	/* So we are beginning... */
     start_time = time(NULL);
 
@@ -549,8 +548,10 @@ static int catsearch(const AFPObj *obj,
 		if (!error && dirpos == NULL)
 			dirpos = opendir(".");
 
-		if (dirpos == NULL)
-			dirpos = opendir(cfrombstr(currentdir->d_fullpath));
+		if (dirpos == NULL) {
+			const char *temp = cfrombstr(currentdir->d_fullpath);
+			dirpos = opendir(temp);
+		}
 
 		if (error || dirpos == NULL) {
 			switch (errno) {
@@ -570,7 +571,7 @@ static int catsearch(const AFPObj *obj,
 			goto catsearch_end;
 		}
 
-		
+
 		while ((entry = readdir(dirpos)) != NULL) {
 			(*pos)++;
 
@@ -595,11 +596,11 @@ static int catsearch(const AFPObj *obj,
 				default:
 					result = AFPERR_MISC;
 					goto catsearch_end;
-				} 
+				}
 			}
             switch (S_IFMT & path.st.st_mode) {
             case S_IFDIR:
-				/* here we can short cut 
+				/* here we can short cut
 				   i.e. if in the same loop the parent dir wasn't in the cache
 				   ALL dirsearch_byname will fail.
 				*/
@@ -619,11 +620,11 @@ static int catsearch(const AFPObj *obj,
 					}
                 }
                 path.m_name = cfrombstr(path.d_dir->d_m_name);
-                	
+
 				if (addstack(path.u_name, path.d_dir, cidx) == -1) {
 					result = AFPERR_MISC;
 					goto catsearch_end;
-				} 
+				}
                 break;
             case S_IFREG:
             	path.d_dir = currentdir;
@@ -637,14 +638,14 @@ static int catsearch(const AFPObj *obj,
 			/* bit 0 means that criteria has been met */
 			if ((ccr & 1)) {
 				r = rslt_add (obj, vol, &path, &rrbuf, ext);
-				
+
 				if (r == 0) {
 					result = AFPERR_MISC;
 					goto catsearch_end;
-				} 
+				}
 				*nrecs += r;
 				/* Number of matches limit */
-				if (--rmatches == 0) 
+				if (--rmatches == 0)
 					goto catsearch_pause;
 				/* Block size limit */
 				if (rrbuf - rbuf >= 448)
@@ -669,14 +670,14 @@ static int catsearch(const AFPObj *obj,
 	goto catsearch_end;
 
 catsearch_pause:
-	cur_pos = *pos; 
+	cur_pos = *pos;
 	save_cidx = cidx;
 
 catsearch_end: /* Exiting catsearch: error condition */
 	*rsize = rrbuf - rbuf;
     if (cwd != -1) {
         if ((fchdir(cwd)) != 0) {
-            LOG(log_debug, logtype_afpd, "error chdiring back: %s", strerror(errno));        
+            LOG(log_debug, logtype_afpd, "error chdiring back: %s", strerror(errno));
         }
         close(cwd);
     }
@@ -700,7 +701,7 @@ catsearch_end: /* Exiting catsearch: error condition */
  */
 static int catsearch_db(const AFPObj *obj,
                         struct vol *vol,
-                        struct dir *dir,  
+                        struct dir *dir _U_,
                         const char *uname,
                         int rmatches,
                         uint32_t *pos,
@@ -721,7 +722,7 @@ static int catsearch_db(const AFPObj *obj,
 
     LOG(log_debug, logtype_afpd, "catsearch_db(req pos: %u): {pos: %u, name: %s}",
         *pos, cur_pos, uname);
-        
+
 	if (*pos != 0 && *pos != cur_pos) {
 		result = AFPERR_CATCHNG;
 		goto catsearch_end;
@@ -755,7 +756,7 @@ static int catsearch_db(const AFPObj *obj,
             goto catsearch_end;
         }
     }
-	
+
 	while (cur_pos < num_matches) {
         char *name;
         cnid_t cnid, did;
@@ -789,11 +790,11 @@ static int catsearch_db(const AFPObj *obj,
             case ELOOP:
                 goto next;
             case ENOENT:
-                
+
             default:
                 result = AFPERR_MISC;
                 goto catsearch_end;
-            } 
+            }
         }
         /* For files path.d_dir is the parent dir, for dirs its the dir itself */
         if (S_ISDIR(path.st.st_mode))
@@ -801,7 +802,7 @@ static int catsearch_db(const AFPObj *obj,
                 goto next;
         path.d_dir = dir;
 
-        LOG(log_maxdebug, logtype_afpd,"catsearch_db: dir: %s, cwd: %s, name: %s", 
+        LOG(log_maxdebug, logtype_afpd,"catsearch_db: dir: %s, cwd: %s, name: %s",
             cfrombstr(dir->d_fullpath), getcwdpath(), path.u_name);
 
         /* At last we can check the search criteria */
@@ -814,10 +815,10 @@ static int catsearch_db(const AFPObj *obj,
             if (r == 0) {
                 result = AFPERR_MISC;
                 goto catsearch_end;
-            } 
+            }
             *nrecs += r;
             /* Number of matches limit */
-            if (--rmatches == 0) 
+            if (--rmatches == 0)
                 goto catsearch_pause;
             /* Block size limit */
             if (rrbuf - rbuf >= 448)
@@ -858,7 +859,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     uint16_t	namelen;
     uint16_t	flags;
     char  	    tmppath[256];
-    char        *uname;
+    const char  *uname = NULL;
 
     *rbuflen = 0;
 
@@ -877,10 +878,10 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     if ((vol = getvolbyvid(vid)) == NULL) {
         return AFPERR_PARAM;
     }
-    
+
     memcpy(&rmatches, ibuf, sizeof(rmatches));
     rmatches = ntohl(rmatches);
-    ibuf += sizeof(rmatches); 
+    ibuf += sizeof(rmatches);
 
     /* FIXME: (rl) should we check if reserved == 0 ? */
     ibuf += sizeof(reserved);
@@ -910,8 +911,8 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     }
     else {
         /* with catsearch only name and parent id are allowed */
-    	c1.fbitmap &= (1<<FILPBIT_LNAME) | (1<<FILPBIT_PDID);
-    	c1.dbitmap &= (1<<DIRPBIT_LNAME) | (1<<DIRPBIT_PDID);
+    	c1.fbitmap &= (1U<<FILPBIT_LNAME) | (1U<<FILPBIT_PDID);
+    	c1.dbitmap &= (1U<<DIRPBIT_LNAME) | (1U<<DIRPBIT_PDID);
         spec_len = *(unsigned char*)ibuf;
     }
 
@@ -919,13 +920,13 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     spec1 = (unsigned char*)ibuf;
     spec2 = (unsigned char*)ibuf + spec_len + 2;
 
-    spec1 += 2; 
-    spec2 += 2; 
+    spec1 += 2;
+    spec2 += 2;
 
     bspec1 = spec1;
     bspec2 = spec2;
     /* File attribute bits... */
-    if (c1.rbitmap & (1 << FILPBIT_ATTR)) {
+    if (c1.rbitmap & (1U << FILPBIT_ATTR)) {
 	    memcpy(&c1.attr, spec1, sizeof(c1.attr));
 	    spec1 += sizeof(c1.attr);
 	    memcpy(&c2.attr, spec2, sizeof(c2.attr));
@@ -933,7 +934,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     }
 
     /* Parent DID */
-    if (c1.rbitmap & (1 << FILPBIT_PDID)) {
+    if (c1.rbitmap & (1U << FILPBIT_PDID)) {
             memcpy(&c1.pdid, spec1, sizeof(pdid));
 	    spec1 += sizeof(c1.pdid);
 	    memcpy(&c2.pdid, spec2, sizeof(pdid));
@@ -941,7 +942,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     } /* FIXME: PDID - do we demarshall this argument ? */
 
     /* Creation date */
-    if (c1.rbitmap & (1 << FILPBIT_CDATE)) {
+    if (c1.rbitmap & (1U << FILPBIT_CDATE)) {
 	    memcpy(&c1.cdate, spec1, sizeof(c1.cdate));
 	    spec1 += sizeof(c1.cdate);
 	    c1.cdate = AD_DATE_TO_UNIX(c1.cdate);
@@ -952,7 +953,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     }
 
     /* Modification date */
-    if (c1.rbitmap & (1 << FILPBIT_MDATE)) {
+    if (c1.rbitmap & (1U << FILPBIT_MDATE)) {
 	    memcpy(&c1.mdate, spec1, sizeof(c1.mdate));
 	    c1.mdate = AD_DATE_TO_UNIX(c1.mdate);
 	    spec1 += sizeof(c1.mdate);
@@ -960,9 +961,9 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
 	    c2.mdate = AD_DATE_TO_UNIX(c2.mdate);
 	    spec2 += sizeof(c1.mdate);
     }
-    
+
     /* Backup date */
-    if (c1.rbitmap & (1 << FILPBIT_BDATE)) {
+    if (c1.rbitmap & (1U << FILPBIT_BDATE)) {
 	    memcpy(&c1.bdate, spec1, sizeof(c1.bdate));
 	    spec1 += sizeof(c1.bdate);
 	    c1.bdate = AD_DATE_TO_UNIX(c1.bdate);
@@ -972,11 +973,11 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     }
 
     /* Finder info */
-    if (c1.rbitmap & (1 << FILPBIT_FINFO)) {
+    if (c1.rbitmap & (1U << FILPBIT_FINFO)) {
     	packed_finder buf;
-    	
+
 	    memcpy(buf, spec1, sizeof(buf));
-	    unpack_buffer(&c1.finfo, buf);    	
+	    unpack_buffer(&c1.finfo, buf);
 	    spec1 += sizeof(buf);
 
 	    memcpy(buf, spec2, sizeof(buf));
@@ -984,7 +985,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
 	    spec2 += sizeof(buf);
     } /* Finder info */
 
-    if ((c1.rbitmap & (1 << DIRPBIT_OFFCNT)) != 0) {
+    if ((c1.rbitmap & (1U << DIRPBIT_OFFCNT)) != 0) {
         /* Offspring count - only directories */
 		if (c1.fbitmap == 0) {
 	    	memcpy(&c1.offcnt, spec1, sizeof(c1.offcnt));
@@ -1003,22 +1004,22 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     } /* Offspring count/ressource fork length */
 
     /* Long name */
-    if (c1.rbitmap & (1 << FILPBIT_LNAME)) {
-        /* Get the long filename */	
+    if (c1.rbitmap & (1U << FILPBIT_LNAME)) {
+        /* Get the long filename */
 		memcpy(tmppath, bspec1 + spec1[1] + 1, (bspec1 + spec1[1])[0]);
 		tmppath[(bspec1 + spec1[1])[0]]= 0;
 		len = convert_string ( vol->v_maccharset, CH_UCS2, tmppath, -1, c1.lname, sizeof(c1.lname));
         if (len == (size_t)(-1))
             return AFPERR_PARAM;
 
-#if 0	
+#if 0
 		/* FIXME: do we need it ? It's always null ! */
 		memcpy(c2.lname, bspec2 + spec2[1] + 1, (bspec2 + spec2[1])[0]);
 		c2.lname[(bspec2 + spec2[1])[0]]= 0;
 #endif
     }
         /* UTF8 Name */
-    if (c1.rbitmap & (1 << FILPBIT_PDINFO)) {
+    if (c1.rbitmap & (1U << FILPBIT_PDINFO)) {
 
 		/* offset */
 		memcpy(&namelen, spec1, sizeof(namelen));
@@ -1043,11 +1044,11 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
         if (len == (size_t)(-1))
             return AFPERR_PARAM;
     }
-    
+
     /* Call search */
     *rbuflen = 24;
-    if ((c1.rbitmap & (1 << FILPBIT_PDINFO))
-        && !(c1.rbitmap & (1<<CATPBIT_PARTIAL))
+    if ((c1.rbitmap & (1U << FILPBIT_PDINFO))
+        && !(c1.rbitmap & (1U<<CATPBIT_PARTIAL))
         && (strcmp(vol->v_cnidscheme, "dbd") == 0)
         && (vol->v_flags & AFPVOL_SEARCHDB))
         /* we've got a name and it's a dbd volume, so search CNID database */
@@ -1090,5 +1091,5 @@ int afp_catsearch_ext (AFPObj *obj, char *ibuf, size_t ibuflen,
 }
 
 /* FIXME: we need a clean separation between afp stubs and 'real' implementation */
-/* (so, all buffer packing/unpacking should be done in stub, everything else 
+/* (so, all buffer packing/unpacking should be done in stub, everything else
    should be done in other functions) */
