@@ -11,36 +11,37 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <arpa/inet.h>
+#include <ctype.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
-#include <sys/stat.h>
-
-#include <errno.h>
-
-#include <atalk/adouble.h>
-#include <sys/uio.h>
 #include <sys/param.h>
 #include <sys/socket.h>
-#include <arpa/inet.h>
+#include <sys/stat.h>
+#include <sys/uio.h>
+#include <netatalk/at.h>
 
-#include <atalk/dsi.h>
+#include <atalk/adouble.h>
 #include <atalk/afp.h>
-#include <atalk/util.h>
-#include <atalk/logger.h>
+#include <atalk/atp.h>
+#include <atalk/asp.h>
+#include <atalk/bstradd.h>
+#include <atalk/bstrlib.h>
+#include <atalk/dsi.h>
+#include <atalk/errchk.h>
 #include <atalk/globals.h>
+#include <atalk/logger.h>
 #include <atalk/netatalk_conf.h>
 #include <atalk/unix.h>
-#include <atalk/bstrlib.h>
-#include <atalk/bstradd.h>
-#include <atalk/errchk.h>
+#include <atalk/util.h>
 
-#include "volume.h"
+#include "desktop.h"
 #include "directory.h"
 #include "fork.h"
-#include "desktop.h"
 #include "mangle.h"
+#include "volume.h"
 
 #define EXEC_MODE (S_IXGRP | S_IXUSR | S_IXOTH)
 
@@ -158,7 +159,7 @@ int setdeskowner(const struct vol *vol, uid_t uid, gid_t gid)
     bcatcstr(dtpath, "/" APPLEDESKTOP);
 
     EC_NEG1( chdir(cfrombstr(dtpath)) );
-    
+
     if (( desk = opendir( "." )) == NULL ) {
         if ( chdir( wd ) < 0 ) {
             LOG(log_error, logtype_afpd, "setdeskowner: chdir %s: %s", wd, strerror(errno) );
@@ -271,12 +272,12 @@ int afp_closedt(AFPObj *obj _U_, char *ibuf _U_, size_t ibuflen _U_, char *rbuf
 
 static struct savedt	si = { { 0, 0, 0, 0 }, -1, 0, 0 };
 
-static char *icon_dtfile(struct vol *vol, u_char creator[ 4 ])
+static char *icon_dtfile(struct vol *vol, uint8_t creator[ 4 ])
 {
     return dtfile( vol, creator, ".icon" );
 }
 
-static int iconopen(struct vol *vol, u_char creator[ 4 ], int flags, int mode)
+static int iconopen(struct vol *vol, uint8_t creator[ 4 ], int flags, int mode)
 {
     char	*dtf, *adt, *adts;
 
@@ -323,8 +324,11 @@ static int iconopen(struct vol *vol, u_char creator[ 4 ], int flags, int mode)
 
 int afp_addicon(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t *rbuflen)
 {
-    struct vol		*vol;
-    u_char		fcreator[ 4 ], imh[ 12 ], irh[ 12 ], *p;
+    struct vol* vol;
+#ifndef NO_DDP
+    struct iovec	iov[2];
+#endif
+    uint8_t		fcreator[4], imh[12], irh[12], * p;
     int			itype, cc = AFP_OK, iovcnt = 0;
     size_t 		buflen;
     uint32_t           ftype, itag;
@@ -414,34 +418,72 @@ int afp_addicon(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
      */
 addicon_err:
     if ( cc < 0 ) {
-        dsi_writeinit(obj->dsi, rbuf, buflen);
-        dsi_writeflush(obj->dsi);
-        return cc;
+        if (obj->proto == AFPPROTO_DSI) {
+            dsi_writeinit(obj->dsi, rbuf, buflen);
+            dsi_writeflush(obj->dsi);
+        }
     }
 
-    DSI *dsi = obj->dsi;
+    switch (obj->proto) {
+#ifndef NO_DDP
+    case AFPPROTO_ASP:
+        buflen = bsize;
+        if ((asp_wrtcont(obj->handle, rbuf, &buflen) < 0) || buflen != bsize)
+            return(AFPERR_PARAM);
 
-    iovcnt = dsi_writeinit(dsi, rbuf, buflen);
+        /*
+         * We're at the end of the file, add the headers, etc.  */
+        if (cc == 0) {
+            iov[0].iov_base = (caddr_t)imh;
+            iov[0].iov_len = sizeof(imh);
+            iov[1].iov_base = rbuf;
+            iov[1].iov_len = bsize;
+            iovcnt = 2;
+        }
 
-    /* add headers at end of file */
-    if ((cc == 0) && (write(si.sdt_fd, imh, sizeof(imh)) < 0)) {
-        LOG(log_error, logtype_afpd, "afp_addicon(%s): write: %s", icon_dtfile(vol, fcreator), strerror(errno));
-        dsi_writeflush(dsi);
-        return AFPERR_PARAM;
-    }
+        /*
+         * We found an icon to replace.
+         */
+        if (cc > 0) {
+            iov[0].iov_base = rbuf;
+            iov[0].iov_len = bsize;
+            iovcnt = 1;
+        }
 
-    if ((cc = write(si.sdt_fd, rbuf, iovcnt)) < 0) {
-        LOG(log_error, logtype_afpd, "afp_addicon(%s): write: %s", icon_dtfile(vol, fcreator), strerror(errno));
-        dsi_writeflush(dsi);
-        return AFPERR_PARAM;
-    }
+        if (writev(si.sdt_fd, iov, iovcnt) < 0) {
+            LOG(log_error, logtype_afpd, "afp_addicon(%s): writev: %s", icon_dtfile(vol, fcreator), strerror(errno));
+            return(AFPERR_PARAM);
+        }
+        break;
+#endif /* no afp/asp */  
+    case AFPPROTO_DSI:
+    {
+        DSI* dsi = obj->dsi;
+
+        iovcnt = dsi_writeinit(dsi, rbuf, buflen);
+
+        /* add headers at end of file */
+        if ((cc == 0) && (write(si.sdt_fd, imh, sizeof(imh)) < 0)) {
+            LOG(log_error, logtype_afpd, "afp_addicon(%s): write: %s", icon_dtfile(vol, fcreator), strerror(errno));
+            dsi_writeflush(dsi);
+            return AFPERR_PARAM;
+        }
 
-    while ((iovcnt = dsi_write(dsi, rbuf, buflen))) {
         if ((cc = write(si.sdt_fd, rbuf, iovcnt)) < 0) {
             LOG(log_error, logtype_afpd, "afp_addicon(%s): write: %s", icon_dtfile(vol, fcreator), strerror(errno));
             dsi_writeflush(dsi);
             return AFPERR_PARAM;
         }
+
+        while ((iovcnt = dsi_write(dsi, rbuf, buflen))) {
+            if ((cc = write(si.sdt_fd, rbuf, iovcnt)) < 0) {
+                LOG(log_error, logtype_afpd, "afp_addicon(%s): write: %s", icon_dtfile(vol, fcreator), strerror(errno));
+                dsi_writeflush(dsi);
+                return AFPERR_PARAM;
+            }
+        }
+    }
+    break;
     }
 
     close( si.sdt_fd );
@@ -449,47 +491,11 @@ addicon_err:
     return( AFP_OK );
 }
 
-static const u_char	utag[] = { 0, 0, 0, 0 };
-static const u_char	ucreator[] = { 0, 0, 0, 0 };/* { 'U', 'N', 'I', 'X' };*/
-static const u_char	utype[] = { 0, 0, 0, 0 };/* { 'T', 'E', 'X', 'T' };*/
+static const uint8_t	utag[] = { 0, 0, 0, 0 };
+static const uint8_t	ucreator[] = { 0, 0, 0, 0 };/* { 'U', 'N', 'I', 'X' };*/
+static const uint8_t	utype[] = { 0, 0, 0, 0 };/* { 'T', 'E', 'X', 'T' };*/
 static const short	usize = 256;
 
-#if 0
-static const u_char	uicon[] = {
-    0x1F, 0xFF, 0xFC, 0x00, 0x10, 0x00, 0x06, 0x00,
-    0x10, 0x00, 0x05, 0x00, 0x10, 0x00, 0x04, 0x80,
-    0x10, 0x00, 0x04, 0x40, 0x10, 0x00, 0x04, 0x20,
-    0x10, 0x00, 0x07, 0xF0, 0x10, 0x00, 0x00, 0x10,
-    0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10,
-    0x10, 0x00, 0x00, 0x10, 0x10, 0x80, 0x02, 0x10,
-    0x11, 0x80, 0x03, 0x10, 0x12, 0x80, 0x02, 0x90,
-    0x12, 0x80, 0x02, 0x90, 0x14, 0x80, 0x02, 0x50,
-    0x14, 0x87, 0xC2, 0x50, 0x14, 0x58, 0x34, 0x50,
-    0x14, 0x20, 0x08, 0x50, 0x12, 0x16, 0xD0, 0x90,
-    0x11, 0x01, 0x01, 0x10, 0x12, 0x80, 0x02, 0x90,
-    0x12, 0x9C, 0x72, 0x90, 0x14, 0x22, 0x88, 0x50,
-    0x14, 0x41, 0x04, 0x50, 0x14, 0x49, 0x24, 0x50,
-    0x14, 0x55, 0x54, 0x50, 0x14, 0x5D, 0x74, 0x50,
-    0x14, 0x5D, 0x74, 0x50, 0x12, 0x49, 0x24, 0x90,
-    0x12, 0x22, 0x88, 0x90, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFC, 0x00, 0x1F, 0xFF, 0xFE, 0x00,
-    0x1F, 0xFF, 0xFF, 0x00, 0x1F, 0xFF, 0xFF, 0x80,
-    0x1F, 0xFF, 0xFF, 0xC0, 0x1F, 0xFF, 0xFF, 0xE0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-    0x1F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF0,
-};
-#endif
 
 int afp_geticoninfo(AFPObj *obj _U_, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t *rbuflen)
 {
@@ -567,7 +573,7 @@ int afp_geticon(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
     struct vol	*vol;
     off_t       offset;
     ssize_t	rc, buflen;
-    u_char	fcreator[ 4 ], ftype[ 4 ], itype, ih[ 12 ];
+    uint8_t	fcreator[ 4 ], ftype[ 4 ], itype, ih[ 12 ];
     uint16_t	vid, bsize, rsize;
 
     buflen = *rbuflen;
@@ -589,16 +595,6 @@ int afp_geticon(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
     memcpy( &bsize, ibuf, sizeof( bsize ));
     bsize = ntohs( bsize );
 
-#if 0
-    if ( memcmp( fcreator, ucreator, sizeof( ucreator )) == 0 &&
-            memcmp( ftype, utype, sizeof( utype )) == 0 &&
-            itype == 1 &&
-            bsize <= usize ) {
-        memcpy( rbuf, uicon, bsize);
-        *rbuflen = bsize;
-        return( AFP_OK );
-    }
-#endif
 
     if ( iconopen( vol, fcreator, O_RDONLY, 0 ) < 0) {
         return( AFPERR_NOITEM );
@@ -643,7 +639,7 @@ int afp_geticon(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
 #define min(a,b)	((a)<(b)?(a):(b))
     rc = min( bsize, rsize );
 
-    if (buflen < rc) {
+    if ((obj->proto == AFPPROTO_DSI) && buflen < rc) {
         DSI *dsi = obj->dsi;
         struct stat st;
         off_t size;
@@ -688,7 +684,7 @@ int afp_geticon(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
 
             *rbuflen = buflen;
         }
-        
+
         dsi_readdone(dsi);
         return AFP_OK;
 
@@ -709,7 +705,7 @@ geticon_exit:
 
 /* ---------------------- */
 static const char		hexdig[] = "0123456789abcdef";
-char *dtfile(const struct vol *vol, u_char creator[], char *ext )
+char *dtfile(const struct vol *vol, uint8_t creator[], char *ext )
 {
     static char	path[ MAXPATHLEN + 1];
     char	*p;
@@ -744,7 +740,7 @@ char *dtfile(const struct vol *vol, u_char creator[], char *ext )
 }
 
 /* ---------------------------
- * mpath is only a filename 
+ * mpath is only a filename
  * did filename parent directory ID.
 */
 
@@ -755,7 +751,7 @@ char *mtoupath(const struct vol *vol, char *mpath, cnid_t did, int utf8)
     size_t       inplen;
     size_t       outlen;
     uint16_t	 flags;
-        
+
     if ( *mpath == '\0' ) {
         strcpy(upath, ".");
         return upath;
@@ -763,7 +759,7 @@ char *mtoupath(const struct vol *vol, char *mpath, cnid_t did, int utf8)
 
     /* set conversion flags */
     flags = vol->v_mtou_flags;
-    
+
     m = demangle(vol, mpath, did);
     if (m != mpath) {
         return m;
@@ -786,7 +782,7 @@ char *mtoupath(const struct vol *vol, char *mpath, cnid_t did, int utf8)
     return( upath );
 }
 
-/* --------------- 
+/* ---------------
  * id filename ID
 */
 char *utompath(const struct vol *vol, char *upath, cnid_t id, int utf8)
@@ -804,7 +800,7 @@ char *utompath(const struct vol *vol, char *upath, cnid_t id, int utf8)
     u = upath;
 
     /* convert charsets */
-    if ((size_t)-1 == ( outlen = convert_charset ( vol->v_volcharset, (utf8)?CH_UTF8_MAC:vol->v_maccharset, vol->v_maccharset, u, outlen, mpath, MAXPATHLEN, &flags)) ) { 
+    if ((size_t)-1 == ( outlen = convert_charset ( vol->v_volcharset, (utf8)?CH_UTF8_MAC:vol->v_maccharset, vol->v_maccharset, u, outlen, mpath, MAXPATHLEN, &flags)) ) {
         LOG(log_error, logtype_afpd, "Conversion from %s to %s for %s (%u) failed.", vol->v_volcodepage, vol->v_maccodepage, u, ntohl(id));
 	goto utompath_error;
     }
@@ -836,14 +832,14 @@ static int ad_addcomment(const AFPObj *obj, struct vol *vol, struct path *path,
     int			clen;
     struct adouble	ad, *adp;
 
-    clen = (u_char)*ibuf++;
+    clen = (uint8_t)*ibuf++;
     clen = min( clen, 199 );
 
     upath = path->u_name;
     if (check_access(obj, vol, upath, OPENACC_WR ) < 0) {
         return AFPERR_ACCESS;
     }
-    
+
     isadir = path_isadir(path);
     if (isadir || !(of = of_findname(vol, path))) {
         ad_init(&ad, vol);
@@ -902,7 +898,7 @@ int afp_addcomment(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_,
 	return get_afp_errno(AFPERR_NOOBJ);
     }
 
-    if ((u_long)ibuf & 1 ) {
+    if ((intptr_t)ibuf & 1 ) {
         ibuf++;
     }
 
@@ -925,7 +921,7 @@ static int ad_getcomment(struct vol *vol, struct path *path, char *rbuf, size_t
         adp = &ad;
     } else
         adp = of->of_ad;
-        
+
     if ( ad_metadata( upath, ((isadir) ? ADFLAGS_DIR : 0), adp) < 0 ) {
         return( AFPERR_NOITEM );
     }
@@ -960,7 +956,7 @@ int afp_getcomment(AFPObj *obj _U_, char *ibuf, size_t ibuflen _U_, char *rbuf,
     struct path         *s_path;
     uint32_t		did;
     uint16_t		vid;
-    
+
     *rbuflen = 0;
     ibuf += 2;
 
@@ -1049,6 +1045,6 @@ int afp_rmvcomment(AFPObj *obj _U_, char *ibuf, size_t ibuflen _U_, char *rbuf _
     if (NULL == ( s_path = cname( vol, dir, &ibuf ))) {
 	return get_afp_errno(AFPERR_NOOBJ);
     }
-    
+
     return ad_rmvcomment(obj, vol, s_path);
 }
