@@ -7,39 +7,42 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
-
 #include <string.h>
-#include <utime.h>
-#include <errno.h>
 #include <sys/param.h>
+#include <utime.h>
 
 #include <atalk/adouble.h>
-#include <atalk/vfs.h>
-#include <atalk/logger.h>
 #include <atalk/afp.h>
-#include <atalk/util.h>
 #include <atalk/cnid.h>
-#include <atalk/unix.h>
-#include <atalk/globals.h>
-#include <atalk/fce_api.h>
-#include <atalk/netatalk_conf.h>
-#include <atalk/spotlight.h>
 #include <atalk/dsi.h>
+#include <atalk/fce_api.h>
+#include <atalk/globals.h>
+#include <atalk/logger.h>
+#include <atalk/netatalk_conf.h>
+
+#ifdef WITH_SPOTLIGHT
+#include <atalk/spotlight.h>
+#endif
+
+#include <atalk/unix.h>
+#include <atalk/util.h>
+#include <atalk/vfs.h>
 
-#include "directory.h"
-#include "dircache.h"
 #include "desktop.h"
-#include "volume.h"
-#include "fork.h"
+#include "dircache.h"
+#include "directory.h"
 #include "file.h"
 #include "filedir.h"
+#include "fork.h"
 #include "unix.h"
+#include "volume.h"
 
 /* the format for the finderinfo fields (from IM: Toolbox Essentials):
  * field         bytes        subfield    bytes
- * 
+ *
  * files:
  * ioFlFndrInfo  16      ->       type    4  type field
  *                             creator    4  creator field
@@ -47,29 +50,29 @@
  *					     alias, bundle, etc.
  *                            location    4  location in window
  *                              folder    2  window that contains file
- * 
+ *
  * ioFlXFndrInfo 16      ->     iconID    2  icon id
- *                              unused    6  reserved 
+ *                              unused    6  reserved
  *                              script    1  script system
  *                              xflags    1  reserved
  *                           commentID    2  comment id
  *                           putawayID    4  home directory id
  */
 
-const u_char ufinderi[ADEDLEN_FINDERI] = {
+const uint8_t ufinderi[ADEDLEN_FINDERI] = {
                               0, 0, 0, 0, 0, 0, 0, 0,
                               1, 0, 0, 0, 0, 0, 0, 0,
                               0, 0, 0, 0, 0, 0, 0, 0,
                               0, 0, 0, 0, 0, 0, 0, 0
                           };
 
-static const u_char old_ufinderi[] = {
+static const uint8_t old_ufinderi[] = {
                               'T', 'E', 'X', 'T', 'U', 'N', 'I', 'X'
                           };
 
-/* ---------------------- 
+/* ----------------------
 */
-static int default_type(void *finder) 
+static int default_type(void *finder)
 {
     if (!memcmp(finder, ufinderi, 8) || !memcmp(finder, old_ufinderi, 8))
         return 1;
@@ -89,7 +92,7 @@ void *get_finderinfo(const struct vol *vol, const char *upath, struct adouble *a
     if (ad_finder) {
         memcpy(data, ad_finder, ADEDLEN_FINDERI);
         /* default type ? */
-        if (default_type(ad_finder)) 
+        if (default_type(ad_finder))
             chk_ext = 1;
     }
     else {
@@ -97,7 +100,7 @@ void *get_finderinfo(const struct vol *vol, const char *upath, struct adouble *a
         chk_ext = 1;
         if (vol_inv_dots(vol) && *upath == '.') { /* make it invisible */
             uint16_t ashort;
-            
+
             ashort = htons(FINDERINFO_INVISIBLE);
             memcpy((char *)data + FINDERINFO_FRFLAGOFF, &ashort, sizeof(ashort));
         }
@@ -108,8 +111,8 @@ void *get_finderinfo(const struct vol *vol, const char *upath, struct adouble *a
         memcpy(&linkflag, (char *)data + FINDERINFO_FRFLAGOFF, 2);
         linkflag |= htons(FINDERINFO_ISALIAS);
         memcpy((char *)data + FINDERINFO_FRFLAGOFF, &linkflag, 2);
-        memcpy((char *)data + FINDERINFO_FRTYPEOFF,"slnk",4); 
-        memcpy((char *)data + FINDERINFO_FRCREATOFF,"rhap",4); 
+        memcpy((char *)data + FINDERINFO_FRTYPEOFF,"slnk",4);
+        memcpy((char *)data + FINDERINFO_FRCREATOFF,"rhap",4);
     }
 
     /** Only enter if no appledouble information and no finder information found. */
@@ -123,7 +126,7 @@ void *get_finderinfo(const struct vol *vol, const char *upath, struct adouble *a
 
 /* ---------------------
 */
-char *set_name(const struct vol *vol, char *data, cnid_t pid, char *name, cnid_t id, uint32_t utf8) 
+char *set_name(const struct vol *vol, char *data, cnid_t pid, char *name, cnid_t id, uint32_t utf8)
 {
     uint32_t   aint;
     char        *tp = NULL;
@@ -135,7 +138,7 @@ char *set_name(const struct vol *vol, char *data, cnid_t pid, char *name, cnid_t
         if (utf8_encoding(vol->v_obj)) {
             /* but name is an utf8 mac name */
             char *u, *m;
-           
+
             /* global static variable... */
             tp = strdup(name);
             if (!(u = mtoupath(vol, name, pid, 1)) || !(m = utompath(vol, u, id, 0))) {
@@ -145,7 +148,7 @@ char *set_name(const struct vol *vol, char *data, cnid_t pid, char *name, cnid_t
                 aint = strlen(m);
                 src = m;
             }
-            
+
         }
         if (aint > MACFILELEN)
             aint = MACFILELEN;
@@ -160,7 +163,7 @@ char *set_name(const struct vol *vol, char *data, cnid_t pid, char *name, cnid_t
         utf8 = vol->v_kTextEncoding;
         memcpy(data, &utf8, sizeof(utf8));
         data += sizeof(utf8);
-        
+
         temp = htons(aint);
         memcpy(data, &temp, sizeof(temp));
         data += sizeof(temp);
@@ -194,8 +197,7 @@ char *set_name(const struct vol *vol, char *data, cnid_t pid, char *name, cnid_t
  *
  * 1. Get the objects CNID as stored in its adouble file
  * 2. Get the objects CNID from the database
- * 3. If there's a problem with a "dbd" database, fallback to "tdb" in memory
- * 4. In case 2 and 3 differ, store 3 in the adouble file
+ * 3. Store resource fork data
  *
  * @param vol    (rw) volume
  * @param adp    (rw) adouble struct of object upath, might be NULL
@@ -205,17 +207,15 @@ char *set_name(const struct vol *vol, char *data, cnid_t pid, char *name, cnid_t
  * @param len    (r) strlen of upath
  */
 uint32_t get_id(struct vol *vol,
-                struct adouble *adp, 
+                struct adouble *adp,
                 const struct stat *st,
                 const cnid_t did,
                 const char *upath,
-                const int len) 
+                const int len)
 {
-    static int first = 1;       /* mark if this func is called the first time */
     uint32_t adcnid;
     uint32_t dbcnid = CNID_INVALID;
 
-restart:
     if (vol->v_cdb != NULL) {
         /* prime aint with what we think is the cnid, set did to zero for
            catching moved files */
@@ -230,6 +230,14 @@ restart:
             switch (errno) {
             case CNID_ERR_CLOSE: /* the db is closed */
                 break;
+            case CNID_ERR_DB:
+                LOG(log_error, logtype_afpd,
+                    "get_id: Connection to the CNID backend DB failed. "
+                    "This is now treated as a fatal error. "
+                    "Is the cnid_metad process running? "
+                    "Please escalate to the upstream project "
+                    "if you suspect this is a bug");
+                exit(EXITERR_SYS);
             case CNID_ERR_PARAM:
                 LOG(log_error, logtype_afpd, "get_id: Incorrect parameters passed to cnid_add");
                 afp_errno = AFPERR_PARAM;
@@ -238,40 +246,11 @@ restart:
                 afp_errno = AFPERR_PARAM;
                 goto exit;
             default:
-                /* Close CNID backend if "dbd" and switch to temp in-memory "tdb" */
-                /* we have to do it here for "dbd" because it uses "lazy opening" */
-                /* In order to not end in a loop somehow with goto restart below  */
-                /*  */
-                if (first && (strcmp(vol->v_cnidscheme, "dbd") == 0)) { /* (3) */
-                    cnid_close(vol->v_cdb);
-                    free(vol->v_cnidscheme);
-                    vol->v_cnidscheme = strdup("tdb");
-
-                    int flags = CNID_FLAG_MEMORY;
-                    if ((vol->v_flags & AFPVOL_NODEV)) {
-                        flags |= CNID_FLAG_NODEV;
-                    }
-                    LOG(log_error, logtype_afpd, "Reopen volume %s using in memory temporary CNID DB.",
-                        vol->v_path);
-                    vol->v_cdb = cnid_open(vol, "tdb", flags);
-                    if (vol->v_cdb) {
-                        if (!(vol->v_flags & AFPVOL_TM)) {
-                            vol->v_flags |= AFPVOL_RO;
-                            setmessage("Something wrong with the volume's CNID DB, using temporary CNID DB instead."
-                                       "Check server messages for details. Switching to read-only mode.");
-                            kill(getpid(), SIGUSR2);
-                        }
-                        goto restart; /* now try again with the temp CNID db */
-                    } else {
-                        setmessage("Something wrong with the volume's CNID DB, using temporary CNID DB failed too!"
-                                   "Check server messages for details, can't recover from this state!");
-                    }
-                }
                 afp_errno = AFPERR_MISC;
                 goto exit;
             }
         }
-        else if (adp && adcnid && (adcnid != dbcnid)) { /* 4 */
+        else if (adp && adcnid && (adcnid != dbcnid)) { /* (3) */
             /* Update the ressource fork. For a folder adp is always null */
             LOG(log_debug, logtype_afpd, "get_id(%s/%s): calling ad_setid(old: %u, new: %u)",
                 getcwdpath(), upath, htonl(adcnid), htonl(dbcnid));
@@ -283,15 +262,14 @@ restart:
     }
 
 exit:
-    first = 0;
     return dbcnid;
 }
-             
+
 /* -------------------------- */
 int getmetadata(const AFPObj *obj,
                 struct vol *vol,
                 uint16_t bitmap,
-                struct path *path, struct dir *dir, 
+                struct path *path, struct dir *dir,
                 char *buf, size_t *buflen, struct adouble *adp)
 {
     char		*data, *l_nameoff = NULL, *upath;
@@ -301,7 +279,7 @@ int getmetadata(const AFPObj *obj,
     uint32_t		aint;
     cnid_t              id = 0;
     uint16_t		ashort;
-    u_char              achar, fdType[4];
+    uint8_t              achar, fdType[4];
     uint32_t           utf8 = 0;
     struct stat         *st;
     struct maccess	ma;
@@ -335,7 +313,7 @@ int getmetadata(const AFPObj *obj,
                         return AFPERR_MISC;
                     }
                 }
-                
+
                 /* Build fullpath */
                 if (((fullpath = bstrcpy(dir->d_fullpath)) == NULL)
                     || (bconchar(fullpath, '/') != BSTR_OK)
@@ -539,7 +517,7 @@ int getmetadata(const AFPObj *obj,
             }
             break;
         case FILPBIT_EXTDFLEN:
-            aint = htonl(st->st_size >> 32);
+            aint = htonl((uint64_t)st->st_size >> 32);
             memcpy(data, &aint, sizeof( aint ));
             data += sizeof( aint );
             aint = htonl(st->st_size);
@@ -548,7 +526,7 @@ int getmetadata(const AFPObj *obj,
             break;
         case FILPBIT_EXTRFLEN:
             if (adp) {
-                aint = htonl(adp->ad_rlen >> 32);
+                aint = htonl((uint64_t)adp->ad_rlen >> 32);
                 memcpy(data, &aint, sizeof( aint ));
                 data += sizeof( aint );
                 aint = htonl(adp->ad_rlen);
@@ -572,7 +550,7 @@ int getmetadata(const AFPObj *obj,
             data += sizeof( aint );
 
 	    /* FIXME: ugly hack
-               type == slnk indicates an OSX style symlink, 
+               type == slnk indicates an OSX style symlink,
                we have to add S_IFLNK to the mode, otherwise
                10.3 clients freak out. */
 
@@ -602,7 +580,7 @@ int getmetadata(const AFPObj *obj,
             *data++ = ma.ma_group;
             *data++ = ma.ma_owner;
             break;
-            
+
         default :
             return( AFPERR_BITMAP );
         }
@@ -622,14 +600,14 @@ int getmetadata(const AFPObj *obj,
     *buflen = data - buf;
     return (AFP_OK);
 }
-                
+
 /* ----------------------- */
 int getfilparams(const AFPObj *obj, struct vol *vol, uint16_t bitmap, struct path *path,
                  struct dir *dir, char *buf, size_t *buflen, int in_enumerate)
 {
     struct adouble	ad, *adp;
     int                 opened = 0;
-    int rc;    
+    int rc;
     int flags; /* uninitialized ok */
 
     LOG(log_debug, logtype_afpd, "getfilparams(\"%s\")", path->u_name);
@@ -684,7 +662,7 @@ int afp_createfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_,
     int			creatf, did, openf, retvalue = AFP_OK;
     uint16_t		vid;
     struct path		*s_path;
-    
+
     *rbuflen = 0;
     ibuf++;
     creatf = (unsigned char) *ibuf++;
@@ -711,7 +689,7 @@ int afp_createfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_,
 
     upath = s_path->u_name;
     ad_init(&ad, vol);
-    
+
     /* if upath is deleted we already in trouble anyway */
     if ((of = of_findname(vol, s_path))) {
         if (creatf)
@@ -825,7 +803,7 @@ int afp_setfilparams(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_
         return( AFPERR_NOOBJ );
     }
 
-    if ((u_long)ibuf & 1 ) {
+    if ((intptr_t)ibuf & 1 ) {
         ibuf++;
     }
 
@@ -837,8 +815,8 @@ int afp_setfilparams(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_
 }
 
 /*
- * cf AFP3.0.pdf page 252 for change_mdate and change_parent_mdate logic  
- * 
+ * cf AFP3.0.pdf page 252 for change_mdate and change_parent_mdate logic
+ *
 */
 extern struct path Cur_Path;
 
@@ -850,12 +828,15 @@ int setfilparams(const AFPObj *obj, struct vol *vol,
     int			bit, isad = 1, err = AFP_OK;
     char                *upath;
     char		*ade = NULL;
-    u_char              achar, *fdType, xyy[4]; /* uninitialized, OK 310105 */
-    uint16_t		ashort, bshort, oshort;
+    uint8_t              achar, xyy[4];
+    const uint8_t        *fdType = NULL;
+    uint16_t		ashort = 0;
+    uint16_t		bshort;
+    uint16_t		oshort;
     uint32_t		aint;
-    uint32_t		upriv;
+    uint32_t		upriv = 0;
     uint16_t           upriv_bit = 0;
-        struct utimbuf	ut;
+    struct utimbuf	ut;
     int                 change_mdate = 0;
     int                 change_parent_mdate = 0;
     int                 newdate = 0;
@@ -864,7 +845,7 @@ int setfilparams(const AFPObj *obj, struct vol *vol,
     gid_t		f_gid;
     uint16_t           bitmap = f_bitmap;
     uint32_t           cdate,bdate;
-    u_char              finder_buf[32];
+    uint8_t              finder_buf[32];
     int symlinked = S_ISLNK(path->st.st_mode);
     int fp;
     ssize_t len;
@@ -890,7 +871,9 @@ int setfilparams(const AFPObj *obj, struct vol *vol,
         }
         switch(  bit ) {
         case FILPBIT_ATTR :
-            change_mdate = 1;
+            /* Preserve file dates during a GS/OS folder copy */
+            if (obj->afp_version >= 30)
+                change_mdate = 1;
             memcpy(&ashort, buf, sizeof( ashort ));
             buf += sizeof( ashort );
             break;
@@ -974,10 +957,10 @@ int setfilparams(const AFPObj *obj, struct vol *vol,
                 buf += 2;
                 /* Keep special case to support crlf translations */
                 if ((unsigned int) achar == 0x04) {
-	       	    fdType = (u_char *)"TEXT";
+	       	    fdType = (uint8_t *)"TEXT";
 		    buf += 2;
                 } else {
-            	    xyy[0] = ( u_char ) 'p';
+            	    xyy[0] = ( uint8_t ) 'p';
             	    xyy[1] = achar;
             	    xyy[3] = *buf++;
             	    xyy[2] = *buf++;
@@ -997,7 +980,7 @@ int setfilparams(const AFPObj *obj, struct vol *vol,
         bit++;
     }
 
-    /* second try with adouble open 
+    /* second try with adouble open
     */
     if (ad_open(adp, upath, ADFLAGS_HF | ADFLAGS_RDWR | ADFLAGS_CREATE, 0666) < 0) {
         LOG(log_debug, logtype_afpd, "setfilparams: ad_open_metadata error: %s", strerror(errno));
@@ -1021,7 +1004,7 @@ int setfilparams(const AFPObj *obj, struct vol *vol,
         }
         (void)ad_setid(adp, path->st.st_dev, path->st.st_ino, id, curdir->d_did, vol->v_stamp);
     }
-    
+
     bit = 0;
     bitmap = f_bitmap;
     while ( bitmap != 0 ) {
@@ -1071,7 +1054,7 @@ int setfilparams(const AFPObj *obj, struct vol *vol,
                 break;
             }
             if (default_type(ade)
-                    && ( 
+                    && (
                      ((em = getextmap( path->m_name )) &&
                       !memcmp(finder_buf, em->em_type, sizeof( em->em_type )) &&
                       !memcmp(finder_buf + 4, em->em_creator,sizeof( em->em_creator)))
@@ -1142,7 +1125,7 @@ setfilparam_done:
  *
  * sdir_fd     source dir fd to which src path is relative (for openat et al semantics)
  *             passing -1 means this is not used, src path is a full path
- * src         the source path 
+ * src         the source path
  * dst         the dest filename in current dir
  * newname     the dest mac name
  * adp         adouble struct of src file, if open, or & zeroed one
@@ -1165,7 +1148,7 @@ int renamefile(struct vol *vol, struct dir *ddir, int sdir_fd, char *src, char *
         case EROFS:
             return AFPERR_VLOCK;
         case EXDEV :			/* Cross device move -- try copy */
-           /* NOTE: with open file it's an error because after the copy we will 
+           /* NOTE: with open file it's an error because after the copy we will
             * get two files, it's fixable for our process (eg reopen the new file, get the
             * locks, and so on. But it doesn't solve the case with a second process
             */
@@ -1185,13 +1168,13 @@ int renamefile(struct vol *vol, struct dir *ddir, int sdir_fd, char *src, char *
 
     if (vol->vfs->vfs_renamefile(vol, sdir_fd, src, dst) < 0 ) {
         int err;
-        
-        err = errno;        
+
+        err = errno;
 	/* try to undo the data fork rename,
-	 * we know we are on the same device 
+	 * we know we are on the same device
 	*/
 	if (err) {
-        unix_rename(-1, dst, sdir_fd, src ); 
+        unix_rename(-1, dst, sdir_fd, src );
     	    /* return the first error */
     	    switch ( err) {
             case ENOENT :
@@ -1217,7 +1200,7 @@ int renamefile(struct vol *vol, struct dir *ddir, int sdir_fd, char *src, char *
     return( AFP_OK );
 }
 
-/* ---------------- 
+/* ----------------
    convert a Mac long name to an utf8 name,
 */
 size_t mtoUTF8(const struct vol *vol, const char *src, size_t srclen, char *dest, size_t destlen)
@@ -1246,7 +1229,7 @@ uint32_t   hint;
     case 2:
         if (( plen = (unsigned char)*ibuf++ ) != 0 ) {
             if (vol->v_obj->afp_version >= 30) {
-                /* convert it to UTF8 
+                /* convert it to UTF8
                 */
                 if ((plen = mtoUTF8(vol, ibuf, plen, newname, AFPOBJ_TMPSIZ)) == (size_t)-1)
                    return -1;
@@ -1257,7 +1240,7 @@ uint32_t   hint;
             }
             if (strlen(newname) != plen) {
                 /* there's \0 in newname, e.g. it's a pathname not
-                 * only a filename. 
+                 * only a filename.
                 */
             	return -1;
             }
@@ -1266,11 +1249,11 @@ uint32_t   hint;
     case 3:
         memcpy(&hint, ibuf, sizeof(hint));
         ibuf += sizeof(hint);
-           
+
         memcpy(&len16, ibuf, sizeof(len16));
         ibuf += sizeof(len16);
         plen = ntohs(len16);
-        
+
         if (plen) {
             if (plen > AFPOBJ_TMPSIZ) {
             	return -1;
@@ -1300,7 +1283,7 @@ int afp_copyfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_, si
 
     struct adouble ad, *adp;
     int denyreadset;
-    
+
     *rbuflen = 0;
     ibuf += 2;
 
@@ -1352,7 +1335,7 @@ int afp_copyfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_, si
         goto copy_exit;
     }
 #endif
-    denyreadset = (ad_testlock(adp, ADEID_DFORK, AD_FILELOCK_DENY_RD) != 0 || 
+    denyreadset = (ad_testlock(adp, ADEID_DFORK, AD_FILELOCK_DENY_RD) != 0 ||
                   ad_testlock(adp, ADEID_RFORK, AD_FILELOCK_DENY_RD) != 0 );
 
     if (denyreadset) {
@@ -1388,7 +1371,7 @@ int afp_copyfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_, si
         retvalue = get_afp_errno(AFPERR_NOOBJ);
         goto copy_exit;
     }
-    
+
     if ( *s_path->m_name != '\0' ) {
 	retvalue =path_error(s_path, AFPERR_NOOBJ);
         goto copy_exit;
@@ -1423,11 +1406,11 @@ copy_exit:
 /* ----------------------------------
  * if newname is NULL (from directory.c) we don't want to copy the resource fork.
  * because we are doing it elsewhere.
- * currently if newname is NULL then adp is NULL. 
+ * currently if newname is NULL then adp is NULL.
  */
 int copyfile(struct vol *s_vol,
-             struct vol *d_vol, 
-             struct dir *d_dir, 
+             struct vol *d_vol,
+             struct dir *d_dir,
              int sfd,
              char *src,
              char *dst,
@@ -1439,7 +1422,7 @@ int copyfile(struct vol *s_vol,
     int                 adflags;
     int                 stat_result;
     struct stat         st;
-    
+
     LOG(log_debug, logtype_afpd, "copyfile(sfd:%d,s:'%s',d:'%s',n:'%s')",
         sfd, src, dst, newname);
 
@@ -1465,7 +1448,7 @@ int copyfile(struct vol *s_vol,
 
     stat_result = fstat(ad_data_fileno(adp), &st); /* saving stat exit code, thus saving us on one more stat later on */
 
-    if (stat_result < 0) {           
+    if (stat_result < 0) {
       /* unlikely but if fstat fails, the default file mode will be 0666. */
       st.st_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
     }
@@ -1480,10 +1463,18 @@ int copyfile(struct vol *s_vol,
         }
         return AFPERR_EXIST;
     }
+    size_t copybuf_len = 0;
+    char* copybuf;
+
+    if (s_vol->v_obj->proto == AFPPROTO_DSI)
+    {
+        copybuf = s_vol->v_obj->dsi->commands;
+        copybuf_len = s_vol->v_obj->dsi->server_quantum;
+    }
 
     if (copy_fork(ADEID_DFORK, &add, adp,
-                         s_vol->v_obj->dsi->commands,
-                         s_vol->v_obj->dsi->server_quantum) != 0) {
+                         copybuf,
+                         copybuf_len) != 0) {
         err = errno;
         LOG(log_error, logtype_afpd, "copyfile('%s'): copy_fork: %s", src, strerror(errno));
     } else if (d_vol->vfs->vfs_copyfile(d_vol, sfd, src, dst) != AFP_OK) {
@@ -1514,7 +1505,7 @@ error:
 
     if (ad_close( &add, adflags ) <0) {
        err = errno;
-    } 
+    }
 
     if (err) {
         deletefile(d_vol, -1, dst, 0);
@@ -1556,7 +1547,7 @@ done:
 
    when deletefile is called we don't have lock on it, file is closed (for us)
    untrue if called by renamefile
-   
+
    ad_open always try to open file RDWR first and ad_lock takes care of
    WRITE lock on read only file.
 */
@@ -1577,7 +1568,7 @@ uint16_t   bshort = 0;
 	}
 	return 0;
 }
-/* 
+/*
  * dirfd can be used for unlinkat semantics
  */
 int deletefile(const struct vol *vol, int dirfd, char *file, int checkAttrib)
@@ -1592,8 +1583,8 @@ int deletefile(const struct vol *vol, int dirfd, char *file, int checkAttrib)
     ad_init(&ad, vol);
     if (checkAttrib) {
         /* was EACCESS error try to get only metadata */
-        /* we never want to create a resource fork here, we are going to delete it 
-         * moreover sometimes deletefile is called with a no existent file and 
+        /* we never want to create a resource fork here, we are going to delete it
+         * moreover sometimes deletefile is called with a no existent file and
          * ad_open would create a 0 byte resource fork
         */
         if ( ad_metadataat(dirfd, file, ADFLAGS_CHECK_OF, &ad) == 0 ) {
@@ -1604,7 +1595,7 @@ int deletefile(const struct vol *vol, int dirfd, char *file, int checkAttrib)
             meta = 1;
         }
     }
- 
+
     /* try to open both forks at once */
     adflags = ADFLAGS_DF;
     if (ad_openat(&ad, dirfd, file, adflags | ADFLAGS_RF | ADFLAGS_NORF | ADFLAGS_RDONLY) < 0 ) {
@@ -1628,11 +1619,11 @@ int deletefile(const struct vol *vol, int dirfd, char *file, int checkAttrib)
 
     if ( adp && AD_RSRC_OPEN(adp) ) { /* there's a resource fork */
         adflags |= ADFLAGS_RF;
-        /* FIXME we have a pb here because we want to know if a file is open 
+        /* FIXME we have a pb here because we want to know if a file is open
          * there's a 'priority inversion' if you can't open the ressource fork RW
          * you can delete it if it's open because you can't get a write lock.
-         * 
-         * ADLOCK_FILELOCK means the whole ressource fork, not only after the 
+         *
+         * ADLOCK_FILELOCK means the whole ressource fork, not only after the
          * metadatas
          *
          * FIXME it doesn't work for RFORK open read only and fork open without deny mode
@@ -1757,13 +1748,13 @@ static int reenumerate_loop(struct dirent *de, char *mname _U_, void *data)
 {
     struct path   path;
     struct reenum *param = data;
-    struct vol    *vol = param->vol;  
+    struct vol    *vol = param->vol;
     cnid_t        did  = param->did;
     cnid_t	  aint;
-    
+
     if (ostat(de->d_name, &path.st, vol_syml_opt(vol)) < 0)
         return 0;
-    
+
     /* update or add to cnid */
     AFP_CNID_START("cnid_add");
     aint = cnid_add(vol->v_cdb, &path.st, did, de->d_name, strlen(de->d_name), 0); /* ignore errors */
@@ -1782,11 +1773,11 @@ reenumerate_id(struct vol *vol, char *name, struct dir *dir)
     int             ret;
     struct reenum   data;
     struct stat     st;
-    
+
     if (vol->v_cdb == NULL) {
 	return -1;
     }
-    
+
     /* FIXME use of_statdir ? */
     if (ostat(name, &st, vol_syml_opt(vol))) {
 	return -1;
@@ -1796,7 +1787,7 @@ reenumerate_id(struct vol *vol, char *name, struct dir *dir)
         /* we already did it once and the dir haven't been modified */
     	return dir->d_offcnt;
     }
-    
+
     data.vol = vol;
     data.did = dir->d_did;
     if ((ret = for_each_dirent(vol, name, reenumerate_loop, (void *)&data)) >= 0) {
@@ -1840,7 +1831,7 @@ int afp_resolveid(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_
     memcpy(&id, ibuf, sizeof( id ));
     ibuf += sizeof(id);
     cnid = id;
-    
+
     if (!id) {
         /* some MacOS versions after a catsearch do a *lot* of afp_resolveid with 0 */
         return AFPERR_NOID;
@@ -1876,7 +1867,7 @@ retry:
 	if (errno == ESTALE) {
 	    errno = ENOENT;
 	}
-#endif	
+#endif
 	if ( errno == ENOENT && !retry) {
 	    /* cnid db is out of sync, reenumerate the directory and update ids */
 	    reenumerate_id(vol, ".", dir);
@@ -1907,7 +1898,7 @@ retry:
         return AFPERR_NOID;
     }
     path.id = cnid;
-    if (AFP_OK != (err = getfilparams(obj, vol, bitmap, &path , curdir, 
+    if (AFP_OK != (err = getfilparams(obj, vol, bitmap, &path , curdir,
                                       rbuf + sizeof(bitmap), &buflen, 0))) {
         return err;
     }
@@ -1975,7 +1966,7 @@ int afp_deleteid(AFPObj *obj _U_, char *ibuf, size_t ibuflen _U_, char *rbuf _U_
             return AFPERR_ACCESS;
 #ifdef ESTALE
 	case ESTALE:
-#endif	
+#endif
         case ENOENT:
             /* still try to delete the id */
             err = AFPERR_NOOBJ;
@@ -2032,7 +2023,7 @@ static struct adouble *find_adouble(const AFPObj *obj, struct vol *vol, struct p
         afp_errno = AFPERR_ACCESS;
         return NULL;
     }
-    
+
     if ((*of = of_findname(vol, path))) {
         /* reuse struct adouble so it won't break locks */
         adp = (*of)->of_ad;
@@ -2063,6 +2054,7 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     char                *supath, *upath;
     struct path         *path;
     int                 err;
+    int                 fd;
     struct adouble	ads;
     struct adouble	add;
     struct adouble	*adsp = NULL;
@@ -2070,8 +2062,9 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     struct ofork	*s_of = NULL;
     struct ofork	*d_of = NULL;
     int                 crossdev;
-    
-    int                 slen, dlen;
+
+    unsigned int        slen;
+    unsigned int        dlen;
     uint32_t		sid, did;
     uint16_t		vid;
 
@@ -2100,7 +2093,7 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     }
 
     if (NULL == ( path = cname( vol, dir, &ibuf )) ) {
-        return get_afp_errno(AFPERR_NOOBJ); 
+        return get_afp_errno(AFPERR_NOOBJ);
     }
 
     if ( path_isadir(path) ) {
@@ -2119,14 +2112,14 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
         /* pathname too long */
         return AFPERR_PARAM ;
     }
-    
+
     ad_init(&ads, vol);
     if (!(adsp = find_adouble(obj, vol, path, &s_of, &ads))) {
         return afp_errno;
     }
 
     /* ***** from here we may have resource fork open **** */
-    
+
     /* look for the source cnid. if it doesn't exist, don't worry about
      * it. */
     AFP_CNID_START("cnid_lookup");
@@ -2139,7 +2132,7 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     }
 
     if (NULL == ( path = cname( vol, dir, &ibuf )) ) {
-        err = get_afp_errno(AFPERR_NOOBJ); 
+        err = get_afp_errno(AFPERR_NOOBJ);
         goto err_exchangefile;
     }
 
@@ -2164,7 +2157,7 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
 
     /* they are not on the same device and at least one is open
      * FIXME broken for for crossdev and adouble v2
-     * return an error 
+     * return an error
     */
     crossdev = (srcst.st_dev != destst.st_dev);
     if (/* (d_of || s_of)  && */ crossdev) {
@@ -2182,7 +2175,6 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
      * NOTE: the temp file will be in the dest file's directory. it
      * will also be inaccessible from AFP. */
     memcpy(temp, APPLETEMP, sizeof(APPLETEMP));
-    int fd;
     if ((fd = mkstemp(temp)) == -1) {
         err = AFPERR_MISC;
         goto err_exchangefile;
@@ -2210,12 +2202,12 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
         goto err_dest_to_src;
     of_rename(vol, s_of, curdir, temp, curdir, path->m_name);
 
-    /* 
-     * id's need switching. src -> dest and dest -> src. 
+    /*
+     * id's need switching. src -> dest and dest -> src.
      * we need to re-stat() if it was a cross device copy.
      */
     if (sid) {
-        AFP_CNID_START("cnid_delete");        
+        AFP_CNID_START("cnid_delete");
         cnid_delete(vol->v_cdb, sid);
         AFP_CNID_DONE();
     }
@@ -2225,7 +2217,7 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
         AFP_CNID_DONE();
     }
 
-    if ((did && ( (crossdev && ostat(upath, &srcst, vol_syml_opt(vol)) < 0) || 
+    if ((did && ( (crossdev && ostat(upath, &srcst, vol_syml_opt(vol)) < 0) ||
                 cnid_update(vol->v_cdb, did, &srcst, curdir->d_did,upath, dlen) < 0))
        ||
         (sid && ( (crossdev && ostat(p, &destst, vol_syml_opt(vol)) < 0) ||
@@ -2277,14 +2269,14 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     }
 
     /* FIXME: we should switch ressource fork too */
-    
+
     /* here we need to reopen if crossdev */
-    if (sid && ad_setid(addp, destst.st_dev, destst.st_ino,  sid, sdir->d_did, vol->v_stamp)) 
+    if (sid && ad_setid(addp, destst.st_dev, destst.st_ino,  sid, sdir->d_did, vol->v_stamp))
     {
        ad_flush( addp );
     }
-        
-    if (did && ad_setid(adsp, srcst.st_dev, srcst.st_ino,  did, curdir->d_did, vol->v_stamp)) 
+
+    if (did && ad_setid(adsp, srcst.st_dev, srcst.st_ino,  did, curdir->d_did, vol->v_stamp))
     {
        ad_flush( adsp );
     }
