@@ -36,14 +36,18 @@
 #include "unix.h"
 
 #ifdef HAVE_LIBQUOTA
-#include <quota/quota.h>
+#include <quota.h>
+
 
 static int
 getfreespace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolSpace *btotal,
-	     uid_t uid, const char *classq)
+	     id_t id, int idtype)
 {
-	int retq;
-	struct ufs_quota_entry ufsq[QUOTA_NLIMITS];
+	uid_t prevuid;
+	const char *msg;
+	struct quotahandle *qh;
+	struct quotakey qk;
+	struct quotaval qv;
 	time_t now;
 
 	if (time(&now) == -1) {
@@ -54,62 +58,94 @@ getfreespace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolSpace *btot
 
     become_root();
 
-	if ((retq = getfsquota(obj, vol, ufsq, uid, classq)) < 0) {
-		LOG(log_info, logtype_afpd, "getfsquota(%s, %s): %s",
-		    vol->v_path, classq, strerror(errno));
+	/*
+	 * In a tidier world we might keep the quotahandle open for longer...
+	 */
+	qh = quota_open(vol->v_path);
+	if (qh == NULL) {
+		if (errno == EOPNOTSUPP || errno == ENXIO) {
+			/* no quotas on this volume */
+			seteuid( prevuid );
+			return 0;
+		}
+
+		LOG(log_info, logtype_afpd, "quota_open(%s): %s", vol->v_path,
+		    strerror(errno));
+		seteuid( prevuid );
+		return -1;
 	}
 
+	qk.qk_idtype = idtype;
+	qk.qk_id = id;
+	qk.qk_objtype = QUOTA_OBJTYPE_BLOCKS;
+	if (quota_get(qh, &qk, &qv) < 0) {
+		if (errno == ENOENT) {
+			/* no quotas for this id */
+			quota_close(qh);
+			seteuid( prevuid );
+			return 0;
+		}
+		msg = strerror(errno);
+		LOG(log_info, logtype_afpd, "quota_get(%s, %s): %s",
+		    vol->v_path, quota_idtype_getname(qh, idtype), msg);
+		quota_close(qh);
+		seteuid( prevuid );
+		return -1;
+	}
+
+	quota_close(qh);
+
     unbecome_root();
 
-	if (retq < 1)
-		return retq;
-
-	switch(QL_STATUS(quota_check_limit(ufsq[QUOTA_LIMIT_BLOCK].ufsqe_cur, 1,
-	    ufsq[QUOTA_LIMIT_BLOCK].ufsqe_softlimit,
-	    ufsq[QUOTA_LIMIT_BLOCK].ufsqe_hardlimit,
-	    ufsq[QUOTA_LIMIT_BLOCK].ufsqe_time, now))) {
-	case QL_S_DENY_HARD:
-	case QL_S_DENY_GRACE:
+	if (qv.qv_usage >= qv.qv_hardlimit ||
+	    (qv.qv_usage >= qv.qv_softlimit && now > qv.qv_expiretime)) {
 		*bfree = 0;
-		*btotal = dbtob(ufsq[QUOTA_LIMIT_BLOCK].ufsqe_cur);
-		break;
-	default:
-		*bfree = dbtob(ufsq[QUOTA_LIMIT_BLOCK].ufsqe_hardlimit -
-		    ufsq[QUOTA_LIMIT_BLOCK].ufsqe_cur);
-		*btotal = dbtob(ufsq[QUOTA_LIMIT_BLOCK].ufsqe_hardlimit);
-		break;
+		*btotal = dbtob(qv.qv_usage);
 	}
+	else {
+		*bfree = dbtob(qv.qv_hardlimit - qv.qv_usage);
+		*btotal = dbtob(qv.qv_hardlimit);
+	}
+
 	return 1;
 }
 
 int uquota_getvolspace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolSpace *btotal, const u_int32_t bsize)
 {
-	int uretq, gretq;
+	int uret, gret;
 	VolSpace ubfree, ubtotal;
 	VolSpace gbfree, gbtotal;
 
-	uretq = getfreespace(obj, vol, &ubfree, &ubtotal,
-			     uuid, QUOTADICT_CLASS_USER);
-	LOG(log_info, logtype_afpd, "getfsquota(%s): %d %d",
-	    vol->v_path, (int)ubfree, (int)ubtotal);
+	uret = getfreespace(obj, vol, &ubfree, &ubtotal,
+	    obj->uid, QUOTA_IDTYPE_USER);
+	if (uret == 1) {
+		LOG(log_info, logtype_afpd, "quota_get(%s, user): %d %d",
+		    vol->v_path, (int)ubfree, (int)ubtotal);
+	}
+
 	if (obj->ngroups >= 1) {
-		gretq = getfreespace(vol, &ubfree, &ubtotal,
-		    obj->groups[0], QUOTADICT_CLASS_GROUP);
+		gret = getfreespace(obj, vol, &gbfree, &gbtotal,
+		    obj->groups[0], QUOTA_IDTYPE_GROUP);
+		if (gret == 1) {
+			LOG(log_info, logtype_afpd, "quota_get(%s, group): %d %d",
+			    vol->v_path, (int)gbfree, (int)gbtotal);
+		}
 	} else
-		gretq = -1;
-	if (uretq < 1 && gretq < 1) { /* no quota for this fs */
+		gret = 0;
+
+	if (uret < 1 && gret < 1) { /* no quota for this fs */
 		return AFPERR_PARAM;
 	}
-	if (uretq < 1) {
-		/* use group quotas */
+	if (uret < 1) {
+		/* no user quotas, but group quotas; use them */
 		*bfree = gbfree;
 		*btotal = gbtotal;
-	} else if (gretq < 1) {
-		/* use user quotas */
+	} else if (gret < 1) {
+		/* no group quotas, but user quotas; use them */
 		*bfree = ubfree;
 		*btotal = ubtotal;
 	} else {
-		/* return smallest remaining space of user and group */
+		/* both; return smallest remaining space of user and group */
 		if (ubfree < gbfree) {
 			*bfree = ubfree;
 			*btotal = ubtotal;
@@ -124,10 +160,6 @@ int uquota_getvolspace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolS
 
 #else /* HAVE_LIBQUOTA */
 
-/*
-#define DEBUG_QUOTA 0
-*/
-
 #define WANT_USER_QUOTA 0
 #define WANT_GROUP_QUOTA 1
 
@@ -140,7 +172,7 @@ int quotactl(int cmd, const char *special, int id, caddr_t addr)
 
 static int overquota( struct dqblk *);
 
-#ifdef linux
+#ifdef __linux__
 
 #ifdef HAVE_LINUX_XQM_H
 #include <linux/xqm.h>
@@ -196,7 +228,7 @@ static void linuxquota_get_api( void )
             kernel_iface = IFACE_VFSV0;
         }
         else if (errno != ENOSYS && errno != ENOTSUP) {
-            /* RedHat 7.1 (2.4.2-2) newquota check 
+            /* RedHat 7.1 (2.4.2-2) newquota check
              * Q_V2_GETSTATS in it's old place, Q_GETQUOTA in the new place
              * (they haven't moved Q_GETSTATS to its new value) */
             int err_stat = 0;
@@ -260,7 +292,7 @@ static int get_linux_quota(int what, char *path, uid_t euser_id, struct dqblk *d
 		r=get_linux_xfs_quota(what, path, euser_id, dp);
 	else
     		r=get_linux_fs_quota(what, path, euser_id, dp);
-    
+
 	return r;
 }
 
@@ -273,7 +305,7 @@ static int get_linux_xfs_quota(int what, char *path, uid_t euser_id, struct dqbl
 	int ret = -1;
 #ifdef HAVE_LINUX_XQM_H
 	struct fs_disk_quota D;
-	
+
 	memset (&D, 0, sizeof(D));
 
 	if ((ret = quotactl(QCMD(Q_XGETQUOTA,(what ? GRPQUOTA : USRQUOTA)), path, euser_id, (caddr_t)&D)))
@@ -285,7 +317,7 @@ static int get_linux_xfs_quota(int what, char *path, uid_t euser_id, struct dqbl
         dqb->dqb_ihardlimit  = (uint64_t)D.d_ino_hardlimit;
         dqb->dqb_isoftlimit  = (uint64_t)D.d_ino_softlimit;
         dqb->dqb_curinodes   = (uint64_t)D.d_icount;
-        dqb->dqb_curblocks   = (uint64_t)D.d_bcount; 
+        dqb->dqb_curblocks   = (uint64_t)D.d_bcount;
 #endif
        return ret;
 }
@@ -403,30 +435,7 @@ mountp( char *file, int *nfs)
 }
 
 #else /* __svr4__ */
-#ifdef ultrix
-/*
-* Return the block-special device name associated with the filesystem
-* on which "file" resides.  Returns NULL on failure.
-*/
-
-static char *
-special( char *file, int *nfs)
-{
-    static struct fs_data	fsd;
-
-    if ( getmnt(0, &fsd, 0, STAT_ONE, file ) < 0 ) {
-        LOG(log_info, logtype_afpd, "special: getmnt %s: %s", file, strerror(errno) );
-        return( NULL );
-    }
-
-    /* XXX: does this really detect an nfs mounted fs? */
-    if (strchr(fsd.fd_req.devname, ':'))
-        *nfs = 1;
-    return( fsd.fd_req.devname );
-}
-
-#else /* ultrix */
-#if (defined(HAVE_SYS_MOUNT_H) && !defined(__linux__)) || defined(BSD4_4) || defined(_IBMR2)
+#if (defined(HAVE_SYS_MOUNT_H) && !defined(__linux__)) || defined(BSD4_4)
 
 static char *
 special(char *file, int *nfs)
@@ -437,14 +446,8 @@ special(char *file, int *nfs)
         return( NULL );
     }
 
-#ifdef TRU64
-    /* Digital UNIX: The struct sfs contains a field sfs.f_type,
-     * the MOUNT_* constants are defined in <sys/mount.h> */
-    if ((sfs.f_type == MOUNT_NFS)||(sfs.f_type == MOUNT_NFS3))
-#else /* TRU64 */
     /* XXX: make sure this really detects an nfs mounted fs */
     if (strchr(sfs.f_mntfromname, ':'))
-#endif /* TRU64 */
         *nfs = 1;
     return( sfs.f_mntfromname );
 }
@@ -490,16 +493,15 @@ special(char *file, int *nfs)
 
     if (!found)
 	return (NULL);
-#ifdef linux
+#ifdef __linux__
     if (strcmp(mnt->mnt_type, "xfs") == 0)
 	is_xfs = 1;
 #endif
-	
+
     return( mnt->mnt_fsname );
 }
 
 #endif /* BSD4_4 */
-#endif /* ultrix */
 #endif /* __svr4__ */
 
 
@@ -512,9 +514,11 @@ static int getfsquota(const AFPObj *obj, struct vol *vol, const int uid, struct
     struct quotctl      qc;
 #endif
 
+#ifdef __linux__
     memset(dq, 0, sizeof(struct dqblk));
+#endif
     memset(&dqg, 0, sizeof(dqg));
-	
+
 #ifdef __svr4__
     qc.op = Q_GETQUOTA;
     qc.uid = uid;
@@ -524,12 +528,6 @@ static int getfsquota(const AFPObj *obj, struct vol *vol, const int uid, struct
     }
 
 #else /* __svr4__ */
-#ifdef ultrix
-    if ( quota( Q_GETDLIM, uid, vol->v_gvs, dq ) != 0 ) {
-        return( AFPERR_PARAM );
-    }
-#else /* ultrix */
-
 #ifndef USRQUOTA
 #define USRQUOTA   0
 #endif
@@ -538,58 +536,49 @@ static int getfsquota(const AFPObj *obj, struct vol *vol, const int uid, struct
 #define QCMD(a,b)  (a)
 #endif
 
-#ifndef TRU64
     /* for group quotas. we only use these if the user belongs
     * to one group. */
-#endif /* TRU64 */
 
 #ifdef BSD4_4
     become_root();
-        if ( quotactl( vol->v_path, QCMD(Q_GETQUOTA,USRQUOTA),
-                       uid, (char *)dq ) != 0 ) {
-            /* try group quotas */
-            if (obj->ngroups >= 1) {
-                if ( quotactl(vol->v_path, QCMD(Q_GETQUOTA, GRPQUOTA),
-                              obj->groups[0], (char *) &dqg) != 0 ) {
-                    unbecome_root();
-                    return( AFPERR_PARAM );
-                }
+    if ( quotactl( vol->v_path, QCMD(Q_GETQUOTA,USRQUOTA),
+                    uid, (char *)dq ) != 0 ) {
+        /* try group quotas */
+        if (obj->ngroups >= 1) {
+            if ( quotactl(vol->v_path, QCMD(Q_GETQUOTA, GRPQUOTA),
+                            obj->groups[0], (char *) &dqg) != 0 ) {
+                unbecome_root();
+                return( AFPERR_PARAM );
             }
         }
-        unbecome_root();
     }
+    unbecome_root();
 
 #else /* BSD4_4 */
     if (get_linux_quota (WANT_USER_QUOTA, vol->v_gvs, uid, dq) !=0) {
-#ifdef DEBUG_QUOTA
         LOG(log_debug, logtype_afpd, "user quota did not work!" );
-#endif /* DEBUG_QUOTA */
     }
 
     if (get_linux_quota(WANT_GROUP_QUOTA, vol->v_gvs, getegid(),  &dqg) != 0) {
-#ifdef DEBUG_QUOTA
         LOG(log_debug, logtype_afpd, "group quota did not work!" );
-#endif /* DEBUG_QUOTA */
 
 	return AFP_OK; /* no need to check user vs group quota */
     }
 #endif  /* BSD4_4 */
 
-
-#ifndef TRU64
     /* return either the group quota entry or user quota entry,
        whichever has the least amount of space remaining
     */
 
     /* if user space remaining > group space remaining */
-    if( 
+    if(
         /* if overquota, free space is 0 otherwise hard-current */
-        ( overquota( dq ) ? 0 : ( dq->dqb_bhardlimit ? dq->dqb_bhardlimit - 
+        ( overquota( dq ) ? 0 : ( dq->dqb_bhardlimit ? dq->dqb_bhardlimit -
                                   dq->dqb_curblocks : ~((uint64_t) 0) ) )
 
       >
-        
-        ( overquota( &dqg ) ? 0 : ( dqg.dqb_bhardlimit ? dqg.dqb_bhardlimit - 
+
+        ( overquota( &dqg ) ? 0 : ( dqg.dqb_bhardlimit ? dqg.dqb_bhardlimit -
                                     dqg.dqb_curblocks : ~((uint64_t) 0) ) )
 
       ) /* if */
@@ -598,17 +587,18 @@ static int getfsquota(const AFPObj *obj, struct vol *vol, const int uid, struct
         dq->dqb_bhardlimit = dqg.dqb_bhardlimit;
         dq->dqb_bsoftlimit = dqg.dqb_bsoftlimit;
         dq->dqb_curblocks = dqg.dqb_curblocks;
+#ifdef __linux__
         dq->dqb_ihardlimit = dqg.dqb_ihardlimit;
         dq->dqb_isoftlimit = dqg.dqb_isoftlimit;
         dq->dqb_curinodes = dqg.dqb_curinodes;
         dq->dqb_btime = dqg.dqb_btime;
         dq->dqb_itime = dqg.dqb_itime;
         dq->bsize = dqg.bsize;
+#else
+        dq->dqb_btimelimit = dqg.dqb_btimelimit;
+#endif
     } /* if */
 
-#endif /* TRU64 */
-
-#endif /* ultrix */
 #endif /* __svr4__ */
 
     return AFP_OK;
@@ -659,43 +649,8 @@ static int getquota(const AFPObj *obj, struct vol *vol, struct dqblk *dq, const
     }
 #endif
 
-#ifdef TRU64
-    /* Digital UNIX: Two forms of specifying an NFS filesystem are possible,
-       either 'hostname:path' or 'path@hostname' (Ultrix heritage) */
-    if (vol->v_nfs) {
-	char *hostpath;
-	char pathstring[MNAMELEN];
-	/* MNAMELEN ist defined in <sys/mount.h> */
-	int result;
-	
-	if ((hostpath = strchr(vol->v_gvs,'@')) != NULL ) {
-	    /* convert 'path@hostname' to 'hostname:path',
-	     * call getnfsquota(),
-	     * convert 'hostname:path' back to 'path@hostname' */
-	    *hostpath = '\0';
-	    sprintf(pathstring,"%s:%s",hostpath+1,vol->v_gvs);
-	    strcpy(vol->v_gvs,pathstring);
-	    
-	    result = getnfsquota(vol, uuid, bsize, dq);
-	    
-	    hostpath = strchr(vol->v_gvs,':');
-	    *hostpath = '\0';
-	    sprintf(pathstring,"%s@%s",hostpath+1,vol->v_gvs);
-	    strcpy(vol->v_gvs,pathstring);
-	    
-	    return result;
-	}
-	else
-	    /* vol->v_gvs is of the form 'hostname:path' */
-	    return getnfsquota(vol, uuid, bsize, dq);
-    } else
-	/* local filesystem */
-      return getfsquota(obj, vol, obj->uid, dq);
-	   
-#else /* TRU64 */
     return vol->v_nfs ? getnfsquota(vol, obj->uid, bsize, dq) :
       getfsquota(obj, vol, obj->uid, dq);
-#endif /* TRU64 */
 }
 
 static int overquota( struct dqblk *dqblk)
@@ -711,11 +666,6 @@ static int overquota( struct dqblk *dqblk)
          dqblk->dqb_bsoftlimit == 0 ) {
         return( 0 );
     }
-#ifdef ultrix
-    if ( dqblk->dqb_bwarn ) {
-        return( 0 );
-    }
-#else /* ultrix */
     if ( gettimeofday( &tv, NULL ) < 0 ) {
         LOG(log_error, logtype_afpd, "overquota: gettimeofday: %s", strerror(errno) );
         return( AFPERR_PARAM );
@@ -723,7 +673,6 @@ static int overquota( struct dqblk *dqblk)
     if ( dqblk->dqb_btimelimit && dqblk->dqb_btimelimit > tv.tv_sec ) {
         return( 0 );
     }
-#endif /* ultrix */
     return( 1 );
 }
 
@@ -749,7 +698,7 @@ static int overquota( struct dqblk *dqblk)
    work */
 #ifdef HAVE_2ARG_DBTOB
 #define tobytes(a, b)  dbtob((VolSpace) (a), (VolSpace) (b))
-#else 
+#else
 #define tobytes(a, b)  dbtob((VolSpace) (a))
 #endif
 
@@ -759,31 +708,15 @@ int uquota_getvolspace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolS
 	struct dqblk dqblk;
 
 	this_bsize = bsize;
-			
+
 	if (getquota(obj, vol, &dqblk, bsize) != 0 ) {
 		return( AFPERR_PARAM );
 	}
 
-#ifdef linux
+#ifdef __linux__
 	this_bsize = dqblk.bsize;
 #endif
 
-#ifdef DEBUG_QUOTA
-        LOG(log_debug, logtype_afpd, "after calling getquota in uquota_getvolspace!" );
-        LOG(log_debug, logtype_afpd, "dqb_ihardlimit: %u", dqblk.dqb_ihardlimit );
-        LOG(log_debug, logtype_afpd, "dqb_isoftlimit: %u", dqblk.dqb_isoftlimit );
-        LOG(log_debug, logtype_afpd, "dqb_curinodes : %u", dqblk.dqb_curinodes );
-        LOG(log_debug, logtype_afpd, "dqb_bhardlimit: %u", dqblk.dqb_bhardlimit );
-        LOG(log_debug, logtype_afpd, "dqb_bsoftlimit: %u", dqblk.dqb_bsoftlimit );
-        LOG(log_debug, logtype_afpd, "dqb_curblocks : %u", dqblk.dqb_curblocks );
-        LOG(log_debug, logtype_afpd, "dqb_btime     : %u", dqblk.dqb_btime );
-        LOG(log_debug, logtype_afpd, "dqb_itime     : %u", dqblk.dqb_itime );
-        LOG(log_debug, logtype_afpd, "bsize/this_bsize : %u/%u", bsize, this_bsize );
-	LOG(log_debug, logtype_afpd, "dqblk.dqb_bhardlimit size: %u", tobytes( dqblk.dqb_bhardlimit, this_bsize ));
-	LOG(log_debug, logtype_afpd, "dqblk.dqb_bsoftlimit size: %u", tobytes( dqblk.dqb_bsoftlimit, this_bsize ));
-	LOG(log_debug, logtype_afpd, "dqblk.dqb_curblocks  size: %u", tobytes( dqblk.dqb_curblocks, this_bsize ));
-#endif /* DEBUG_QUOTA */ 
-
 	/* no limit set for this user. it might be set in the future. */
 	if (dqblk.dqb_bsoftlimit == 0 && dqblk.dqb_bhardlimit == 0) {
         	*btotal = *bfree = ~((VolSpace) 0);
@@ -803,13 +736,6 @@ int uquota_getvolspace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolS
                  	  tobytes( dqblk.dqb_curblocks, this_bsize );
     	}
 
-#ifdef DEBUG_QUOTA
-        LOG(log_debug, logtype_afpd, "bfree          : %u", *bfree );
-        LOG(log_debug, logtype_afpd, "btotal         : %u", *btotal );
-        LOG(log_debug, logtype_afpd, "bfree          : %uKB", *bfree/1024 );
-        LOG(log_debug, logtype_afpd, "btotal         : %uKB", *btotal/1024 );
-#endif
-
 	return( AFP_OK );
 }
 #endif /* HAVE_LIBQUOTA */
