@@ -7,29 +7,9 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <stdbool.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <ctype.h>
-#include <string.h>
-#include <time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <sys/socket.h>
-#include <atalk/logger.h>
-
-#if 0
-#ifdef BSD4_4
-#include <sys/param.h>
-#ifndef HAVE_GETHOSTID
-#include <sys/sysctl.h>
-#endif /* HAVE_GETHOSTID */
-#endif /* BSD4_4 */
-#endif
-
 #include <arpa/inet.h>
+#include <ctype.h>
+#include <fcntl.h>
 
 #ifdef HAVE_KERBEROS
 #ifdef HAVE_KRB5_KRB5_H
@@ -39,14 +19,27 @@
 #endif /* HAVE_KRB5_KRB5_H */
 #endif /* HAVE_KERBEROS */
 
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/socket.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <time.h>
+#include <unistd.h>
+
 #include <atalk/dsi.h>
+#include <atalk/atp.h>
+#include <atalk/asp.h>
+#include <atalk/globals.h>
+#include <atalk/logger.h>
 #include <atalk/unicode.h>
 #include <atalk/util.h>
-#include <atalk/globals.h>
 
-#include "status.h"
 #include "afp_config.h"
 #include "icon.h"
+#include "status.h"
 #include "uam_auth.h"
 
 static   size_t maxstatuslen = 0;
@@ -95,7 +88,7 @@ static void status_flags(char *data,
 static int status_server(char *data, const char *server, const struct afp_options *options)
 {
     char                *start = data;
-    char                *Obj;
+    char*       Obj;
     char		buf[32];
     uint16_t           status;
     size_t		len;
@@ -105,8 +98,9 @@ static int status_server(char *data, const char *server, const struct afp_option
 
     /* extract the obj part of the server */
     Obj = (char *) server;
-    if ((size_t)-1 == (len = convert_string( 
-                           options->unixcharset, options->maccharset, 
+
+    if ((size_t)-1 == (len = convert_string(
+                           options->unixcharset, options->maccharset,
                            Obj, -1, buf, sizeof(buf))) ) {
         len = MIN(strlen(Obj), 31);
     	*data++ = len;
@@ -128,7 +122,7 @@ static int status_server(char *data, const char *server, const struct afp_option
      * signature and net address offsets if they're not going to be
      * used. as there are no offsets after them, it doesn't hurt to
      * have them specified though. so, we just do that to simplify
-     * things.  
+     * things.
      *
      * NOTE2: AFP3.1 Documentation states that the directory names offset
      * is a required feature, even though it can be set to zero.
@@ -144,11 +138,7 @@ static void status_machine(char *data)
     char                *start = data;
     uint16_t           status;
     int			len;
-#ifdef AFS
-    const char		*machine = "afs";
-#else /* !AFS */
     const char		*machine = "Netatalk%s";
-#endif /* AFS */
     char buf[AFPSTATUS_MACHLEN+1];
 
     memcpy(&status, start + AFPSTATUS_MACHOFF, sizeof(status));
@@ -204,6 +194,9 @@ static uint16_t status_signature(char *data, int *servoffset,
 }
 
 static size_t status_netaddress(char *data, int *servoffset,
+#ifndef NO_DDP
+                             const ASP asp,
+#endif
                              const DSI *dsi,
                              const struct afp_options *options)
 {
@@ -218,10 +211,10 @@ static size_t status_netaddress(char *data, int *servoffset,
     data += ntohs(offset);
 
     /* format:
-       Address count (byte) 
-       len (byte = sizeof(length + address type + address) 
+       Address count (byte)
+       len (byte = sizeof(length + address type + address)
        address type (byte, ip address = 0x01, ip + port = 0x02,
-                           ddp address = 0x03, fqdn = 0x04) 
+                           ddp address = 0x03, fqdn = 0x04)
        address (up to 254 bytes, ip = 4, ip + port = 6, ddp = 4)
        */
 
@@ -229,6 +222,9 @@ static size_t status_netaddress(char *data, int *servoffset,
        connection, but we don't have the ip address. to get around this,
        we turn off the status flag for tcp/ip. */
     *data++ = ((options->fqdn && dsi)? 1 : 0) + (dsi ? 1 : 0) +
+#ifndef NO_DDP
+        (asp ? 1 : 0) +
+#endif
               (((options->flags & OPTION_ANNOUNCESSH) && options->fqdn && dsi)? 1 : 0);
 
     /* ip address */
@@ -286,10 +282,10 @@ static size_t status_netaddress(char *data, int *servoffset,
             addresses_len += len+2;
         }
 
-        /* Annouce support for SSH tunneled AFP session, 
+        /* Annouce support for SSH tunneled AFP session,
          * this feature is available since 10.3.2.
          * According to the specs (AFP 3.1 p.225) this should
-         * be an IP+Port style value, but it only works with 
+         * be an IP+Port style value, but it only works with
          * a FQDN. OSX Server uses FQDN as well.
          */
         if ( len + 2 + addresses_len < maxstatuslen - offset) {
@@ -302,8 +298,25 @@ static size_t status_netaddress(char *data, int *servoffset,
         }
     }
 
+#ifndef NO_DDP
+    if (asp) {
+        const struct sockaddr_at* ddpaddr = atp_sockaddr(asp->asp_atp);
+
+        /* ddp address */
+        *data++ = 6;
+        *data++ = 0x03; /* ddp address */
+        memcpy(data, &ddpaddr->sat_addr.s_net, sizeof(ddpaddr->sat_addr.s_net));
+        data += sizeof(ddpaddr->sat_addr.s_net);
+        memcpy(data, &ddpaddr->sat_addr.s_node,
+            sizeof(ddpaddr->sat_addr.s_node));
+        data += sizeof(ddpaddr->sat_addr.s_node);
+        memcpy(data, &ddpaddr->sat_port, sizeof(ddpaddr->sat_port));
+        data += sizeof(ddpaddr->sat_port);
+    }
+#endif /* ! NO_DDP */
+
     /* calculate/store Directory Services Names offset */
-    offset = htons(data - begin); 
+    offset = htons(data - begin);
     *servoffset += sizeof(offset);
     memcpy(begin + *servoffset, &offset, sizeof(offset));
 
@@ -363,7 +376,7 @@ static size_t status_utf8servername(char *data, int *nameoffset,
     LOG(log_info, logtype_afpd, "servername: %s", options->hostname);
 
     if ((len = convert_string(options->unixcharset,
-                              CH_UTF8_MAC, 
+                              CH_UTF8_MAC,
                               options->hostname,
                               -1,
                               data + sizeof(namelen),
@@ -413,15 +426,25 @@ static void status_icon(char *data, const unsigned char *icondata,
 
 /* ---------------------
  */
-void status_init(AFPObj *obj, DSI *dsi)
+void status_init(AFPObj *dsi_obj, AFPObj* asp_obj, DSI *dsi)
 {
+#ifndef NO_DDP
+    ASP asp;
+#endif
     char *status = dsi->status;
     size_t statuslen;
     int c, sigoff, ipok = 0;
-    const struct afp_options *options = &obj->options;
+    const struct afp_options *options = &dsi_obj->options;
 
     maxstatuslen = sizeof(dsi->status);
 
+#ifndef NO_DDP
+    if (asp_obj->handle) {
+        asp = asp_obj->handle;
+    }
+    else
+        asp = NULL;
+#endif
     if (dsi->server.ss_family == AF_INET) { /* IPv4 */
         const struct sockaddr_in *sa4 = (struct sockaddr_in *)&dsi->server;
         ipok = sa4->sin_addr.s_addr ? 1 : 0;
@@ -452,34 +475,50 @@ void status_init(AFPObj *obj, DSI *dsi)
      *
      * at the appropriate offsets:
      * machine type, afp versions, uams, server signature
-     * (16-bytes), network addresses, volume icon/mask 
+     * (16-bytes), network addresses, volume icon/mask
      */
 
     status_flags(status,
                  options->flags & OPTION_SERVERNOTIF,
                  (options->fqdn || ipok),
-                 options->passwdbits, 
+                 options->passwdbits,
                  1,
                  options->flags);
     /* returns offset to signature offset */
     c = status_server(status, options->hostname, options);
     status_machine(status);
-    status_versions(status, dsi);
+    status_versions(status, 
+#ifndef NO_DDP
+                    asp,
+#endif
+                    dsi);
     status_uams(status, options->uamlist);
     status_icon(status, icon, sizeof(icon), c);
 
     sigoff = status_signature(status, &c, options);
     /* c now contains the offset where the netaddress offset lives */
 
-    status_netaddress(status, &c, dsi, options);
+    status_netaddress(status, &c,
+#ifndef NO_DDP
+                      asp,
+#endif
+                      dsi, options);
     /* c now contains the offset where the Directory Names Count offset lives */
 
     statuslen = status_directorynames(status, &c, dsi, options);
     /* c now contains the offset where the UTF-8 ServerName offset lives */
 
-    if ( statuslen < maxstatuslen) 
+    if ( statuslen < maxstatuslen)
         statuslen = status_utf8servername(status, &c, dsi, options);
 
+#ifndef NO_DDP
+    if (asp_obj->handle) {
+        asp_setstatus(asp, status, statuslen);
+        asp_obj->signature = status + sigoff;
+        //asp_obj->statuslen = statuslen;
+    }
+#endif /* ! NO_DDP */
+
     dsi->signature = status + sigoff;
     dsi->statuslen = statuslen;
 }
@@ -501,7 +540,7 @@ void set_signature(struct afp_options *options) {
     FILE *fp = NULL;
     size_t len;
     char *server_tmp;
-    
+
     server_tmp = options->hostname;
     len = strlen(options->signatureopt);
     if (len == 0) {
@@ -517,21 +556,21 @@ void set_signature(struct afp_options *options) {
         LOG(log_info, logtype_afpd, "signature string is %s.", options->signatureopt);
         goto server_signature_user;
     }
-    
+
 server_signature_user:
-    
+
     /* Signature is defined in afp.conf */
     memset(options->signature, 0, 16);
     memcpy(options->signature, options->signatureopt, len);
     goto server_signature_done;
-    
+
 server_signature_auto:
-    
+
     /* Signature type is auto, using afp_signature.conf */
     if (!stat(options->sigconffile, &tmpstat)) {                /* conf file exists? */
         if ((fp = fopen(options->sigconffile, "r")) != NULL) {  /* read open? */
             /* scan in the conf file */
-            while (fgets(buf, sizeof(buf), fp) != NULL) { 
+            while (fgets(buf, sizeof(buf), fp) != NULL) {
                 p = buf;
                 while (p && isblank(*p))
                     p++;
@@ -549,10 +588,10 @@ server_signature_auto:
                 p++;
                 if (*p == '\0')
                     continue;                             /* syntax error: missing signature */
-                
+
                 if (strcmp(server_tmp, servername_conf))
                     continue;                             /* another servername */
-                
+
                 while (p && isblank(*p))
                     p++;
                 if ( 16 == sscanf(p, "%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX",
@@ -578,7 +617,7 @@ server_signature_auto:
                     fseek(fp, -1L, SEEK_END);
                     if(fgetc(fp) != '\n') fputc('\n', fp); /* last char is \n? */
                     goto server_signature_random;
-                }                    
+                }
             } else {
                 LOG(log_error, logtype_afpd, "ERROR: Cannot write in %s (%s). Using one-time signature.",
                     options->sigconffile, strerror(errno));
@@ -604,9 +643,9 @@ server_signature_auto:
         header = 1;
         goto server_signature_random;
     }
-    
+
 server_signature_random:
-    
+
     /* generate signature from random number */
     randombytes(options->signature, 16);
 
@@ -619,7 +658,7 @@ server_signature_random:
         fprintf(fp, "# \n");
         fprintf(fp, "# If setting \"signature = xxxxx\" in afp.conf, this file is not used.\n\n");
     }
-    
+
     if (fp) {
         fprintf(fp, "\"%s\"\t", server_tmp);
         for (i=0 ; i<16 ; i++) {
@@ -628,9 +667,9 @@ server_signature_random:
         fprintf(fp, "%s", "\n");
         fclose(fp);
     }
-    
+
 server_signature_done:
-    
+
     /* retrun */
     LOG(log_info, logtype_afpd,
         "signature is %02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X",
@@ -642,7 +681,7 @@ server_signature_done:
         (options->signature)[10], (options->signature)[11],
         (options->signature)[12], (options->signature)[13],
         (options->signature)[14], (options->signature)[15]);
-    
+
     return;
 }
 
