@@ -35,22 +35,22 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <arpa/inet.h>
 #include <errno.h>
-#include <sys/param.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdarg.h>
-#include <arpa/inet.h>
+#include <sys/param.h>
 
-#include <atalk/logger.h>
 #include <atalk/adouble.h>
-#include <atalk/util.h>
-#include <atalk/unix.h>
-#include <atalk/ea.h>
-#include <atalk/bstrlib.h>
 #include <atalk/bstradd.h>
+#include <atalk/bstrlib.h>
 #include <atalk/compat.h>
+#include <atalk/ea.h>
 #include <atalk/errchk.h>
+#include <atalk/logger.h>
+#include <atalk/unix.h>
+#include <atalk/util.h>
 #include <atalk/volume.h>
 
 #include "ad_lock.h"
@@ -293,7 +293,7 @@ const char *openflags2logstr(int oflags)
         first = 0;
     }
     return buf;
-}    
+}
 
 static uint32_t get_eid(uint32_t eid)
 {
@@ -359,7 +359,7 @@ int ad_init_offsets(struct adouble *ad)
 /* ----------------------------------- */
 static int new_ad_header(struct adouble *ad, const char *path, struct stat *stp, int adflags)
 {
-    const struct entry  *eid;
+    const struct entry  *eid _U_;
     uint16_t            ashort;
     struct stat         st;
     char                *adp = NULL;
@@ -657,7 +657,7 @@ EC_CLEANUP:
 }
 
 /* Read an ._ file, only uses the resofork, finderinfo is taken from EA */
-static int ad_header_read_osx(const char *path, struct adouble *ad, const struct stat *hst)
+static int ad_header_read_osx(const char *path, struct adouble *ad, struct stat *hst)
 {
     EC_INIT;
     struct adouble      adosx;
@@ -668,6 +668,11 @@ static int ad_header_read_osx(const char *path, struct adouble *ad, const struct
     int                 retry_read = 0;
 
 reread:
+    if (hst == NULL) {
+        hst = &st;
+        EC_NEG1( fstat(ad_reso_fileno(ad), hst) );
+    }
+
     LOG(log_debug, logtype_ad, "ad_header_read_osx: %s", path ? fullpathname(path) : "");
     ad_init_old(&adosx, AD_VERSION_EA, ad->ad_options);
     buf = &adosx.ad_data[0];
@@ -707,7 +712,7 @@ reread:
         return -1;
     }
 
-    if (parse_entries(&adosx, nentries, header_len) != 0) {
+    if (parse_entries(&adosx, nentries, hst->st_size) != 0) {
         LOG(log_warning, logtype_ad, "ad_header_read(%s): malformed AppleDouble",
             path ? fullpathname(path) : "");
             errno = EIO;
@@ -725,6 +730,7 @@ reread:
         }
         retry_read++;
         if (ad_convert_osx(path, &adosx) == 1) {
+            hst = NULL;
             goto reread;
         }
         errno = EIO;
@@ -740,11 +746,6 @@ reread:
         return -1;
     }
 
-    if (hst == NULL) {
-        hst = &st;
-        EC_NEG1( fstat(ad_reso_fileno(ad), &st) );
-    }
-
     ad_setentryoff(ad, ADEID_RFORK, ad_getentryoff(&adosx, ADEID_RFORK));
     ad->ad_rlen = hst->st_size - ad_getentryoff(ad, ADEID_RFORK);
 
@@ -756,7 +757,7 @@ static int ad_header_read_ea(const char *path, struct adouble *ad, const struct
 {
     EC_INIT;
     uint16_t nentries;
-    int      len;
+    int      len _U_;
     ssize_t  header_len;
     char     *buf = ad->ad_data;
 
@@ -803,14 +804,6 @@ static int ad_header_read_ea(const char *path, struct adouble *ad, const struct
         EC_FAIL;
     }
 
-    /*
-     * Since recent CVE fixes have introduced new behavior regarding
-     * ad_entry() output. For now, we will AFP_ASSERT() in EC_CLEANUP to prevent
-     * altering on-disk info. This does introduce an avenue to DOS
-     * the netatalk server by locally writing garbage to the EA. At this
-     * point, the outcome is an acceptable risk to prevent unintended
-     * changes to metadata.
-     */
     if (nentries != ADEID_NUM_EA
         || !ad_entry(ad, ADEID_FINDERI)
         || !ad_entry(ad, ADEID_COMMENT)
@@ -820,12 +813,6 @@ static int ad_header_read_ea(const char *path, struct adouble *ad, const struct
         || !ad_entry(ad, ADEID_PRIVINO)
         || !ad_entry(ad, ADEID_PRIVSYN)
         || !ad_entry(ad, ADEID_PRIVID)) {
-        LOG(log_error, logtype_ad,
-            "ad_header_read_ea(\"%s\"): invalid metadata EA "
-            "this is now being treated as a fatal error. "
-            "if you see this log entry, please file a bug ticket "
-            "with your upstream vendor and attach the generated "
-            "core file.", path ? fullpathname(path) : "UNKNOWN");
 
         errno = EINVAL;
         EC_FAIL;
@@ -839,19 +826,17 @@ static int ad_header_read_ea(const char *path, struct adouble *ad, const struct
         ad_setentryoff(ad, ADEID_RFORK, ADEDOFF_RFORK_OSX);
 #endif
 
-    // TODO: Remove the assert and restore the branch for cleaning up invalid metadata
-    //       https://github.com/Netatalk/netatalk/issues/400
 EC_CLEANUP:
-    AFP_ASSERT(!(ret != 0 && errno == EINVAL));
-#if 0
     if (ret != 0 && errno == EINVAL) {
         become_root();
         (void)sys_removexattr(path, AD_EA_META);
         unbecome_root();
-        LOG(log_error, logtype_ad, "ad_header_read_ea(\"%s\"): deleted invalid metadata EA", fullpathname(path), nentries);
+        LOG(log_error, logtype_ad,
+            "ad_header_read_ea(%s, %d): deleted invalid metadata EA",
+            path ? fullpathname(path) : "UNKNOWN", nentries);
+
         errno = ENOENT;
     }
-#endif
     EC_EXIT;
 }
 
@@ -959,7 +944,7 @@ static int ad_header_upgrade_ea(struct adouble *ad _U_, const char *name _U_)
  */
 static int ad_error(struct adouble *ad, int adflags)
 {
-    int err = errno;
+    int err _U_ = errno;
     if (adflags & ADFLAGS_NOHF) { /* 1 */
         return 0;
     }
@@ -1224,7 +1209,7 @@ EC_CLEANUP:
     EC_EXIT;
 }
 
-static int ad_open_hf_ea(const char *path, int adflags, int mode, struct adouble *ad)
+static int ad_open_hf_ea(const char *path, int adflags, int mode _U_, struct adouble *ad)
 {
     EC_INIT;
     int oflags;
@@ -1310,7 +1295,7 @@ EC_CLEANUP:
         ad_data_fileno(ad), ad->ad_data_fork.adf_refcount,
         ad_meta_fileno(ad), ad->ad_mdp->adf_refcount,
         ad_reso_fileno(ad), ad->ad_rfp->adf_refcount);
-        
+
     EC_EXIT;
 }
 
@@ -1343,7 +1328,7 @@ static int ad_open_hf(const char *path, int adflags, int mode, struct adouble *a
 /*!
  * Get resofork length for adouble:ea, parameter 'ad' may be NULL
  */
-off_t ad_reso_size(const char *path, int adflags, struct adouble *ad)
+off_t ad_reso_size(const char *path, int adflags, struct adouble *ad _U_)
 {
     EC_INIT;
     struct stat st;
@@ -1384,7 +1369,7 @@ EC_CLEANUP:
     return rlen;
 }
 
-static int ad_open_rf_v2(const char *path, int adflags, int mode, struct adouble *ad)
+static int ad_open_rf_v2(const char *path, int adflags, int mode _U_, struct adouble *ad)
 {
     EC_INIT;
 
@@ -2083,7 +2068,7 @@ int ad_metadata(const char *name, int flags, struct adouble *adp)
     int   ret, err, oflags;
 
     /* Sanitize flags */
-    oflags = (flags & (ADFLAGS_CHECK_OF | ADFLAGS_DIR)) | ADFLAGS_HF | ADFLAGS_RDONLY;    
+    oflags = (flags & (ADFLAGS_CHECK_OF | ADFLAGS_DIR)) | ADFLAGS_HF | ADFLAGS_RDONLY;
 
     if ((ret = ad_open(adp, name, oflags)) < 0 && errno == EACCES) {
         become_root();
