@@ -1,29 +1,29 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    Samba system utilities
    Copyright (C) Andrew Tridgell 1992-1998
    Copyright (C) Jeremy Allison  1998-2005
    Copyright (C) Timur Bakeyev        2005
    Copyright (C) Bjoern Jacke    2006-2007
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-   
+
    sys_copyxattr modified from LGPL2.1 libattr copyright
    Copyright (C) 2001-2002 Silicon Graphics, Inc.  All Rights Reserved.
    Copyright (C) 2001 Andreas Gruenbacher.
-      
+
    Samba 3.0.28, modified for netatalk.
 */
 
@@ -31,10 +31,10 @@
 #include "config.h"
 #endif
 
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <errno.h>
 
 #if HAVE_ATTR_XATTR_H
 #include <attr/xattr.h>
@@ -56,11 +56,11 @@
 #endif
 
 #include <atalk/adouble.h>
-#include <atalk/util.h>
-#include <atalk/logger.h>
-#include <atalk/ea.h>
 #include <atalk/compat.h>
+#include <atalk/ea.h>
 #include <atalk/errchk.h>
+#include <atalk/logger.h>
+#include <atalk/util.h>
 
 /******** Solaris EA helper function prototypes ********/
 #ifdef SOLARIS
@@ -76,7 +76,7 @@ static int solaris_openat(int fildes, const char *path, int oflag, mode_t mode);
 
 /**************************************************************************
  Wrappers for extented attribute calls. Based on the Linux package with
- support for IRIX and (Net|Free)BSD also. Expand as other systems have them.
+ support for (Net|Free)BSD also. Expand as other systems have them.
 ****************************************************************************/
 static char attr_name[256 +5] = "user.";
 
@@ -90,7 +90,7 @@ static const char *prefix(const char *uname)
 #endif
 }
 
-int sys_getxattrfd(int fd, const char *uname, int oflag, ...)
+int sys_getxattrfd(int fd _U_, const char *uname _U_, int oflag _U_, ...)
 {
 #if defined(SOLARIS)
     int eafd;
@@ -153,7 +153,7 @@ ssize_t sys_getxattr (const char *path, const char *uname, void *value, size_t s
 	int retval, flags = 0;
 	int valuelength = (int)size;
 	char *attrname = strchr(name,'.') + 1;
-	
+
 	if (strncmp(name, "system", 6) == 0) flags |= ATTR_ROOT;
 
 	retval = attr_get(path, attrname, (char *)value, &valuelength, flags);
@@ -189,7 +189,7 @@ ssize_t sys_fgetxattr (int filedes, const char *uname, void *value, size_t size)
 #elif defined(HAVE_EXTATTR_GET_FD)
     char *s;
     ssize_t retval;
-    int attrnamespace = (strncmp(name, "system", 6) == 0) ? 
+    int attrnamespace = (strncmp(name, "system", 6) == 0) ?
         EXTATTR_NAMESPACE_SYSTEM : EXTATTR_NAMESPACE_USER;
     const char *attrname = ((s=strchr(name, '.')) == NULL) ? name : s + 1;
 
@@ -265,7 +265,7 @@ ssize_t sys_lgetxattr (const char *path, const char *uname, void *value, size_t
 	int retval, flags = ATTR_DONTFOLLOW;
 	int valuelength = (int)size;
 	char *attrname = strchr(name,'.') + 1;
-	
+
 	if (strncmp(name, "system", 6) == 0) flags |= ATTR_ROOT;
 
 	retval = attr_get(path, attrname, (char *)value, &valuelength, flags);
@@ -293,7 +293,7 @@ static struct {
         int space;
 	const char *name;
 	size_t len;
-} 
+}
 extattr[] = {
 	{ EXTATTR_NAMESPACE_SYSTEM, EXTATTR_PREFIX("") },
         { EXTATTR_NAMESPACE_USER, EXTATTR_PREFIX("") },
@@ -444,7 +444,7 @@ static ssize_t remove_user(ssize_t ret, char *list, size_t size)
 	char *ptr;
 	char *ptr1;
 	ssize_t ptrsize;
-	
+
 	if (ret <= 0 || size == 0)
 		return ret;
 	ptrsize = ret;
@@ -499,7 +499,7 @@ ssize_t sys_listxattr (const char *path, char *list, size_t size)
 #endif
 }
 
-ssize_t sys_flistxattr (int filedes, const char *path, char *list, size_t size)
+ssize_t sys_flistxattr (int filedes _U_, const char *path, char *list, size_t size)
 {
 #if defined(HAVE_LISTXATTR)
 	ssize_t ret;
@@ -587,7 +587,7 @@ int sys_removexattr (const char *path, const char *uname)
 #elif defined(HAVE_ATTR_REMOVE)
 	int flags = 0;
 	char *attrname = strchr(name,'.') + 1;
-	
+
 	if (strncmp(name, "system", 6) == 0) flags |= ATTR_ROOT;
 
 	return attr_remove(path, attrname, flags);
@@ -605,7 +605,7 @@ int sys_removexattr (const char *path, const char *uname)
 #endif
 }
 
-int sys_fremovexattr (int filedes, const char *path, const char *uname)
+int sys_fremovexattr (int filedes _U_, const char *path, const char *uname)
 {
 	const char *name = prefix(uname);
 #if defined(HAVE_REMOVEXATTR)
@@ -622,7 +622,7 @@ int sys_fremovexattr (int filedes, const char *path, const char *uname)
 #elif defined(HAVE_ATTR_REMOVE)
 	int flags = 0;
 	char *attrname = strchr(name,'.') + 1;
-	
+
 	if (strncmp(name, "system", 6) == 0) flags |= ATTR_ROOT;
 
 	return attr_remove(path, attrname, flags);
@@ -655,7 +655,7 @@ int sys_lremovexattr (const char *path, const char *uname)
 #elif defined(HAVE_ATTR_REMOVE)
 	int flags = ATTR_DONTFOLLOW;
 	char *attrname = strchr(name,'.') + 1;
-	
+
 	if (strncmp(name, "system", 6) == 0) flags |= ATTR_ROOT;
 
 	return attr_remove(path, attrname, flags);
@@ -673,7 +673,7 @@ int sys_lremovexattr (const char *path, const char *uname)
 #endif
 }
 
-int sys_setxattr (const char *path, const char *uname, const void *value, size_t size, int flags)
+int sys_setxattr (const char *path, const char *uname, const void *value, size_t size, int flags _U_)
 {
 	const char *name = prefix(uname);
 #if defined(HAVE_SETXATTR)
@@ -711,7 +711,7 @@ int sys_setxattr (const char *path, const char *uname, const void *value, size_t
 #elif defined(HAVE_ATTR_SET)
 	int myflags = 0;
 	char *attrname = strchr(name,'.') + 1;
-	
+
 	if (strncmp(name, "system", 6) == 0) myflags |= ATTR_ROOT;
 	if (flags & XATTR_CREATE) myflags |= ATTR_CREATE;
 	if (flags & XATTR_REPLACE) myflags |= ATTR_REPLACE;
@@ -735,7 +735,7 @@ int sys_setxattr (const char *path, const char *uname, const void *value, size_t
 #endif
 }
 
-int sys_fsetxattr (int filedes, const char *uname, const void *value, size_t size, int flags)
+int sys_fsetxattr (int filedes, const char *uname, const void *value, size_t size, int flags _U_)
 {
     const char *name = prefix(uname);
 
@@ -751,7 +751,7 @@ int sys_fsetxattr (int filedes, const char *uname, const void *value, size_t siz
 #elif defined(HAVE_EXTATTR_SET_FD)
     char *s;
     int retval = 0;
-    int attrnamespace = (strncmp(name, "system", 6) == 0) ? 
+    int attrnamespace = (strncmp(name, "system", 6) == 0) ?
         EXTATTR_NAMESPACE_SYSTEM : EXTATTR_NAMESPACE_USER;
     const char *attrname = ((s=strchr(name, '.')) == NULL) ? name : s + 1;
     if (flags) {
@@ -801,7 +801,7 @@ int sys_fsetxattr (int filedes, const char *uname, const void *value, size_t siz
 #endif
 }
 
-int sys_lsetxattr (const char *path, const char *uname, const void *value, size_t size, int flags)
+int sys_lsetxattr (const char *path, const char *uname, const void *value, size_t size, int flags _U_)
 {
 	const char *name = prefix(uname);
 #if defined(HAVE_LSETXATTR)
@@ -838,7 +838,7 @@ int sys_lsetxattr (const char *path, const char *uname, const void *value, size_
 #elif defined(HAVE_ATTR_SET)
 	int myflags = ATTR_DONTFOLLOW;
 	char *attrname = strchr(name,'.') + 1;
-	
+
 	if (strncmp(name, "system", 6) == 0) myflags |= ATTR_ROOT;
 	if (flags & XATTR_CREATE) myflags |= ATTR_CREATE;
 	if (flags & XATTR_REPLACE) myflags |= ATTR_REPLACE;
@@ -969,7 +969,7 @@ static int solaris_attropen(const char *path, const char *attrpath, int oflag, m
             EC_FAIL;
         }
 	}
-    
+
 EC_CLEANUP:
     if (filedes != -1)
         close(filedes);
