@@ -127,7 +127,7 @@ static int RF_renamedir_adouble(VFS_FUNC_ARGS_RENAMEDIR)
 }
 
 /* ----------------- */
-static int deletecurdir_adouble_loop(const struct vol *vol, struct dirent *de, char *name, void *data _U_, int flag _U_)
+static int deletecurdir_adouble_loop(const struct vol *vol _U_, struct dirent *de, char *name, void *data _U_, int flag _U_)
 {
     struct stat st;
     int         err;
@@ -480,7 +480,7 @@ static int deletecurdir_ea_osx_chkifempty_loop(const struct vol *vol, struct dir
     return 0;
 }
 
-static int deletecurdir_ea_osx_loop(const struct vol *vol, struct dirent *de, char *name, void *data _U_, int flag _U_)
+static int deletecurdir_ea_osx_loop(const struct vol *vol _U_, struct dirent *de _U_, char *name, void *data _U_, int flag _U_)
 {
     int ret;
     struct stat sb;
@@ -735,7 +735,11 @@ static struct vfs_ops netatalk_adouble_v2 = {
     /* vfs_deletefile:    */ RF_deletefile_adouble,
     /* vfs_renamefile:    */ RF_renamefile_adouble,
     /* vfs_copyfile:      */ RF_copyfile_adouble,
-    NULL
+    /* vfs_ea_getsize:    */ NULL,
+    /* vfs_ea_getcontent: */ NULL,
+    /* vfs_ea_list:       */ NULL,
+    /* vfs_ea_set:        */ NULL,
+    /* vfs_ea_remove:     */ NULL
 };
 
 static struct vfs_ops netatalk_adouble_ea = {
@@ -750,7 +754,11 @@ static struct vfs_ops netatalk_adouble_ea = {
     /* vfs_deletefile:    */ RF_deletefile_ea,
     /* vfs_renamefile:    */ RF_renamefile_ea,
     /* vfs_copyfile:      */ RF_copyfile_ea,
-    NULL
+    /* vfs_ea_getsize:    */ NULL,
+    /* vfs_ea_getcontent: */ NULL,
+    /* vfs_ea_list:       */ NULL,
+    /* vfs_ea_set:        */ NULL,
+    /* vfs_ea_remove:     */ NULL
 };
 
 /*
